@@ -266,7 +266,7 @@ class GitConfig:
     jf_options: dict[str, Any] = field(default_factory=dict)
 
     # Maps when we should pull commits from for each repository (by ['org_login']['repo_id'] name)
-    repos_to_skip_pull_prs_for: dict[str, datetime] = field(default_factory=dict)
+    repos_to_skip_pull_prs_for: dict[str, bool] = field(default_factory=dict)
     repos_to_prs_last_updated: dict[str, datetime] = field(default_factory=dict)
     # Maps the oldest PR/commit date we have in our system
     repos_to_commits_backpopulated_to: dict[str, datetime] = field(default_factory=dict)
@@ -309,9 +309,11 @@ class GitConfig:
     included_branches_by_repo: Dict[str, List[str]] = field(default_factory=dict)
     git_redact_names_and_urls: bool = False
     git_strip_text_content: bool = False
-    check_ghc_mannequin_user_prs: bool = False
     include_pr_comment_reactions: bool = True
 
+    # Legacy field for pulling mannequin user data from GHC
+    check_ghc_mannequin_user_prs: bool = False
+
     def __post_init__(self):
         if type(self.git_provider) == str:
             if self.git_provider.upper() in [
