@@ -21,3 +21,7 @@ class Constants:
     PULL_SPRINTS_SKIP_INACTIVE = 'makara-pull-sprints-skip-inactive-2025Q1'
     # Flag for if we should crawl across all our issues and pull additional users we detect
     CHECK_ISSUES_FOR_EXTRA_USERS_AND_PULL = 'makara-check-for-extra-users-in-issues-2025Q2'
+    # Feature flag for JQL Enhanced Search API migration
+    JQL_ENHANCED_SEARCH_ENABLED = 'makara-jql-enhanced-search-enabled-2025Q3'
+    # Feature flag to force legacy API for troubleshooting
+    FORCE_LEGACY_API = 'makara-force-legacy-api-2025Q3'
