@@ -190,30 +190,17 @@ if __name__ == "__main__":
                 ingest_config.jira_config.jellyfish_issue_ids_for_redownload
             )
             if not skip_custom_field_detect_and_repair:
-                # Calculate API selection decision using the same logic as _run_load_and_push_to_s3()
-                should_attempt_enhanced_search = ingest_config.jira_config.feature_flags.get(
-                    Constants.JQL_ENHANCED_SEARCH_ENABLED, False
-                ) and not ingest_config.jira_config.feature_flags.get(
-                    Constants.FORCE_LEGACY_API, False
+                # Test if JQL Enhanced Search is available
+                use_jql_enhanced_search = is_jql_enhanced_search_available(
+                    jira_config=ingest_config.jira_config,
+                    jql_enhanced_search_enabled=ingest_config.jira_config.feature_flags.get(
+                        Constants.JQL_ENHANCED_SEARCH_ENABLED, False
+                    ),
+                    force_legacy_api=ingest_config.jira_config.feature_flags.get(
+                        Constants.FORCE_LEGACY_API, False
+                    ),
                 )
 
-                # Create test connection with v3 if we should attempt enhanced search
-                use_jql_enhanced_search = False
-                if should_attempt_enhanced_search:
-                    try:
-                        test_connection = get_jira_connection(
-                            config=ingest_config.jira_config, use_jql_enhanced_search=True
-                        )
-                        use_jql_enhanced_search = is_jql_enhanced_search_available(test_connection)
-                    except Exception as e:
-                        logger.warning(
-                            f"Failed to create v3 test connection for JQL Enhanced Search: {e}"
-                        )
-                        use_jql_enhanced_search = False
-
-                api_type = "JQL Enhanced Search" if use_jql_enhanced_search else "Legacy"
-                logger.info(f"Using {api_type} API for custom field detection")
-
                 jcfv_update_payload = identify_custom_field_mismatches(
                     ingest_config=ingest_config, use_jql_enhanced_search=use_jql_enhanced_search
                 )
