@@ -29,6 +29,8 @@ from jf_ingest.jf_git.standardized_models import (
 )
 from jf_ingest.utils import hash_filename
 
+logger = logging.getLogger(__name__)
+
 
 class AzureDevopsAdapter(GitAdapter):
     def __init__(self, config: GitConfig):
@@ -63,7 +65,9 @@ class AzureDevopsAdapter(GitAdapter):
         """
         return not self.client.api_version.startswith('6.1')
 
-    def _project_name_from_repo(self, standardized_repo: StandardizedRepository) -> str:
+    def _project_name_from_repo(
+        self, standardized_repo: StandardizedRepository, force_load_all_repos: bool = False
+    ) -> str:
         '''
         The Project Name should be pre-loaded into the self.repo_id_to_project_name object,
         which gets set in the adapter get_repos function, and mapped to the repo ID. If
@@ -75,10 +79,25 @@ class AzureDevopsAdapter(GitAdapter):
         if project_name := self.repo_id_to_project_name.get(standardized_repo.id):
             return project_name
         else:
-            logging_helper.send_to_agent_log_file(
-                f'Repos were not preloaded? Loading them now', level=logging.WARNING
-            )
-            for repo in self.client.get_all_repos(org_name=standardized_repo.organization.login):
+            # For performance reasons we may want or not want to load all repos. All repos should be loaded
+            # when doing a get_repos call in Git Pull. We don't want to load ALL repos if we are just fetching
+            # one item from one repo, however.
+            if force_load_all_repos:
+                logging_helper.send_to_agent_log_file(
+                    f'Repos were not preloaded? Loading all repos now', level=logging.WARNING
+                )
+                for repo in self.client.get_all_repos(
+                    org_name=standardized_repo.organization.login
+                ):
+                    self.repo_id_to_project_name[repo['id']] = repo['project']['name']
+            else:
+                logging_helper.send_to_agent_log_file(
+                    f'This repo was not preloaded? Loading this repo ({standardized_repo.name}) now',
+                    level=logging.DEBUG,
+                )
+                repo = self.client.get_repo(
+                    org_name=standardized_repo.organization.login, repo_id=standardized_repo.id
+                )
                 self.repo_id_to_project_name[repo['id']] = repo['project']['name']
 
         return self.repo_id_to_project_name[standardized_repo.id]
@@ -104,6 +123,24 @@ class AzureDevopsAdapter(GitAdapter):
                     return
                 i += 1
 
+    def get_user(
+        self, user_identifier: str, standardize: bool = False, org_login: Optional[str] = None
+    ) -> dict | StandardizedUser:
+        assert org_login, "org_login must be provided for ADO"
+        api_user: dict = self.client.get_graph_user(
+            org_name=org_login, user_identifier=user_identifier
+        )
+
+        if standardize:
+            if standardized_user := _standardize_graph_user(api_user):
+                return standardized_user
+            else:
+                raise Exception(
+                    f'Could not standardize user {user_identifier}. Raw data: {api_user}'
+                )
+        else:
+            return api_user
+
     def get_teams(
         self, standardized_organization: StandardizedOrganization, limit: Optional[int] = None
     ) -> Generator[StandardizedTeam, None, None]:
@@ -342,6 +379,138 @@ class AzureDevopsAdapter(GitAdapter):
             chain(abandoned_prs_generator, completed_prs_generator, active_prs),
         )
 
+    def _process_api_pr(
+        self,
+        api_pr: dict,
+        standardized_repo: StandardizedRepository,
+        pull_files_for_pr: bool = False,
+        hash_files_for_prs: bool = False,
+    ) -> StandardizedPullRequest:
+        login = standardized_repo.organization.login
+        repo_id = standardized_repo.id
+        project_name = self._project_name_from_repo(standardized_repo)
+
+        pr_id = api_pr['pullRequestId']
+        head_branch = api_pr['sourceRefName'].replace('refs/heads/', '')
+        base_branch = api_pr['targetRefName'].replace('refs/heads/', '')
+        files = {}
+        changed_count = 0
+        additions = 0
+        deletions = 0
+        if 'lastMergeTargetCommit' in api_pr and 'lastMergeCommit' in api_pr:
+            change_counts = self.client.get_pull_request_changes_counts(
+                org_name=login,
+                repo_id=standardized_repo.id,
+                base_sha=api_pr['lastMergeTargetCommit']['commitId'],
+                target_sha=api_pr['lastMergeCommit']['commitId'],
+            )
+            changed_count = sum(change_counts.values())
+            additions = change_counts.get('Add', 0)
+            deletions = change_counts.get('Delete', 0)
+            if pull_files_for_pr:
+                changes = self.client.get_pull_request_diff(
+                    org_name=login,
+                    repo_id=standardized_repo.id,
+                    base_sha=api_pr['lastMergeTargetCommit']['commitId'],
+                    target_sha=api_pr['lastMergeCommit']['commitId'],
+                )
+                files = {
+                    c['item']['path']: StandardizedFileData(
+                        status=c.get('changedType', ''),
+                        changes=0,
+                        additions=0,
+                        deletions=0,
+                    )
+                    for c in changes
+                    if not c['item'].get('isFolder', False)
+                }
+                if hash_files_for_prs:
+                    files = {
+                        hash_filename(file_path): file_data
+                        for file_path, file_data in files.items()
+                    }
+            else:
+                files = {}
+
+        pr_updated_at = self.get_pr_updated_date(standardized_repo=standardized_repo, api_pr=api_pr)
+
+        commits = [
+            _standardize_commit(
+                api_commit=api_commit,
+                branch_name=head_branch,
+                standardized_repo=standardized_repo,
+            )
+            for api_commit in self.client.get_pr_commits(
+                org_name=login,
+                project_name=project_name,
+                repo_id=repo_id,
+                pr_id=pr_id,
+            )
+        ]
+        # If a merge commit is in here, retroactively mark one of the commits
+        # as a merge commit
+        if api_pr['status'] == 'completed' and 'lastMergeCommit' in api_pr:
+            for commit in commits:
+                if commit.hash == api_pr['lastMergeCommit']['commitId']:
+                    commit.is_merge = True
+
+        comments: List[StandardizedPullRequestComment] = []
+        reviews: List[StandardizedPullRequestReview] = []
+        for comment_thread in self.client.get_pr_comment_threads(
+            org_name=login, project_name=project_name, repo_id=repo_id, pr_id=pr_id
+        ):
+            comment_thread_id = comment_thread['id']
+            for api_comment in comment_thread['comments']:
+                comments.append(_standardize_pull_request_comment(api_comment))
+                pr_review = _standardize_pull_request_review(
+                    api_comment=api_comment,
+                    pr_id=pr_id,
+                    comment_thread_id=comment_thread_id,
+                )
+
+                if pr_review:
+                    reviews.append(pr_review)
+
+        labels = [label['name'] for label in api_pr.get('labels', [])]
+        return _standardize_pull_request(
+            api_pr,
+            standardized_repo,
+            pr_updated_at=pr_updated_at,
+            commits=commits,
+            comments=comments,
+            approvals=reviews,
+            labels=labels,
+            additions=additions,
+            deletions=deletions,
+            changed_count=changed_count,
+            files=files,
+        )
+
+    def get_pr(
+        self, login: str, repo_id: str, pr_number: str, standardize: bool = False
+    ) -> dict | StandardizedPullRequest:
+        """NOTE: Due to heavy processing of ADO PRs, we always standardize PRs when fetching them."""
+
+        standardized_repo = self.get_repo(
+            login=login, repo_id=repo_id, standardize=True
+        )  # type: ignore
+        project_name = self._project_name_from_repo(standardized_repo)  # type: ignore
+        api_pr = self.client.get_pull_request(
+            org_name=login,
+            project_name=project_name,
+            repo_id=repo_id,
+            pr_id=int(pr_number),
+        )
+
+        logger.debug(
+            'When fetching for PRs for ADO we always standardize them due to the complexity of the data involved.'
+        )
+
+        return self._process_api_pr(
+            api_pr=api_pr,
+            standardized_repo=standardized_repo,  # type: ignore
+        )  # type: ignore
+
     def get_prs(
         self,
         standardized_repo: StandardizedRepository,
@@ -353,10 +522,6 @@ class AzureDevopsAdapter(GitAdapter):
         end_window: Optional[datetime] = None,
     ) -> Generator[StandardizedPullRequest, None, None]:
         try:
-            login = standardized_repo.organization.login
-            repo_id = standardized_repo.id
-            project_name = self._project_name_from_repo(standardized_repo)
-
             api_prs = self._get_prs_helper(
                 standardized_repo=standardized_repo,
                 start_window=start_window,
@@ -368,103 +533,11 @@ class AzureDevopsAdapter(GitAdapter):
                 start=1,
             ):
                 try:
-                    pr_id = api_pr['pullRequestId']
-                    head_branch = api_pr['sourceRefName'].replace('refs/heads/', '')
-                    base_branch = api_pr['targetRefName'].replace('refs/heads/', '')
-                    files = {}
-                    changed_count = 0
-                    additions = 0
-                    deletions = 0
-                    if 'lastMergeTargetCommit' in api_pr and 'lastMergeCommit' in api_pr:
-                        change_counts = self.client.get_pull_request_changes_counts(
-                            org_name=login,
-                            repo_id=standardized_repo.id,
-                            base_sha=api_pr['lastMergeTargetCommit']['commitId'],
-                            target_sha=api_pr['lastMergeCommit']['commitId'],
-                        )
-                        changed_count = sum(change_counts.values())
-                        additions = change_counts.get('Add', 0)
-                        deletions = change_counts.get('Delete', 0)
-                        if pull_files_for_pr:
-                            changes = self.client.get_pull_request_diff(
-                                org_name=login,
-                                repo_id=standardized_repo.id,
-                                base_sha=api_pr['lastMergeTargetCommit']['commitId'],
-                                target_sha=api_pr['lastMergeCommit']['commitId'],
-                            )
-                            files = {
-                                c['item']['path']: StandardizedFileData(
-                                    status=c.get('changedType', ''),
-                                    changes=0,
-                                    additions=0,
-                                    deletions=0,
-                                )
-                                for c in changes
-                                if not c['item'].get('isFolder', False)
-                            }
-                            if hash_files_for_prs:
-                                files = {
-                                    hash_filename(file_path): file_data
-                                    for file_path, file_data in files.items()
-                                }
-                        else:
-                            files = {}
-
-                    pr_updated_at = self.get_pr_updated_date(
-                        standardized_repo=standardized_repo, api_pr=api_pr
-                    )
-
-                    commits = [
-                        _standardize_commit(
-                            api_commit=api_commit,
-                            branch_name=head_branch,
-                            standardized_repo=standardized_repo,
-                        )
-                        for api_commit in self.client.get_pr_commits(
-                            org_name=login,
-                            project_name=project_name,
-                            repo_id=repo_id,
-                            pr_id=pr_id,
-                        )
-                    ]
-                    # If a merge commit is in here, retroactively mark one of the commits
-                    # as a merge commit
-                    if api_pr['status'] == 'completed' and 'lastMergeCommit' in api_pr:
-                        for commit in commits:
-                            if commit.hash == api_pr['lastMergeCommit']['commitId']:
-                                commit.is_merge = True
-
-                    comments: List[StandardizedPullRequestComment] = []
-                    reviews: List[StandardizedPullRequestReview] = []
-                    for comment_thread in self.client.get_pr_comment_threads(
-                        org_name=login, project_name=project_name, repo_id=repo_id, pr_id=pr_id
-                    ):
-                        comment_thread_id = comment_thread['id']
-                        for api_comment in comment_thread['comments']:
-                            comments.append(_standardize_pull_request_comment(api_comment))
-                            pr_review = _standardize_pull_request_review(
-                                api_comment=api_comment,
-                                pr_id=pr_id,
-                                comment_thread_id=comment_thread_id,
-                            )
-
-                            if pr_review:
-                                reviews.append(pr_review)
-
-                    labels = [label['name'] for label in api_pr.get('labels', [])]
-
-                    yield _standardize_pull_request(
-                        api_pr,
-                        standardized_repo,
-                        pr_updated_at=pr_updated_at,
-                        commits=commits,
-                        comments=comments,
-                        approvals=reviews,
-                        labels=labels,
-                        additions=additions,
-                        deletions=deletions,
-                        changed_count=changed_count,
-                        files=files,
+                    yield self._process_api_pr(
+                        api_pr=api_pr,
+                        standardized_repo=standardized_repo,
+                        pull_files_for_pr=pull_files_for_pr,
+                        hash_files_for_prs=hash_files_for_prs,
                     )
                     if limit and i >= limit:
                         return
@@ -505,6 +578,91 @@ class AzureDevopsAdapter(GitAdapter):
         """
         raise NotImplementedError()
 
+    def get_repo(
+        self, login: str, repo_id: str, standardize: bool = False
+    ) -> dict | StandardizedRepository:
+        api_repo = self.client.get_repo(
+            org_name=login,
+            repo_id=repo_id,
+        )
+        if standardize:
+            standardized_org = _standardize_organization(
+                org_login=login,
+                url=self.client.base_url,
+            )
+            return _standardize_repo(
+                org=standardized_org,
+                api_repo=api_repo,
+            )
+        else:
+            return api_repo
+
+    def get_commit(
+        self, login: str, repo_id: str, commit_hash: str, standardize: bool = False
+    ) -> dict | StandardizedCommit:
+        repo: StandardizedRepository = self.get_repo(
+            login=login, repo_id=repo_id, standardize=True
+        )  # type: ignore
+
+        project_name = self._project_name_from_repo(standardized_repo=repo)
+        api_commit = self.client.get_commit(
+            org_name=login,
+            project_name=project_name,
+            repo_id=repo_id,
+            commit_hash=commit_hash,
+        )
+        if standardize:
+            # We need to get the repo object to standardize the commit
+            api_repo = self.client.get_repo(
+                org_name=login,
+                repo_id=repo_id,
+            )
+            standardized_org = _standardize_organization(
+                org_login=login,
+                url=self.client.base_url,
+            )
+            standardized_repo = _standardize_repo(
+                org=standardized_org,
+                api_repo=api_repo,
+            )
+            return _standardize_commit(
+                api_commit=api_commit,
+                branch_name=None,
+                standardized_repo=standardized_repo,
+            )
+        else:
+            return api_commit
+
+    def get_branch(
+        self, login: str, repo_id: str, branch_name: str, standardize: bool = False
+    ) -> dict | StandardizedBranch:
+        """Get a branch by name"""
+
+        standardized_repo = self.get_repo(
+            login=login,
+            repo_id=repo_id,
+            standardized=True,
+        )  # type: ignore
+
+        api_branch = self.client.get_branch(
+            org_name=login,
+            project_name=self._project_name_from_repo(standardized_repo),  # type: ignore
+            repo_id=repo_id,
+            branch_name=branch_name,
+        )
+        if standardize:
+            if branch := _standardize_branch(
+                api_branch=api_branch,
+                standardized_repo=standardized_repo,  # type: ignore
+            ):
+                return branch
+            else:
+                raise Exception(
+                    f'Could not standardize branch {branch_name} from repo {repo_id}. Raw data: {api_branch}'
+                )
+        else:
+            return api_branch
+
 
 '''
 
