@@ -126,6 +126,16 @@ class GithubAdapter(GitAdapter):
             if limit and i >= limit:
                 return
 
+    def get_user(
+        self, user_identifier: str, standardize: bool = False, org_login: Optional[str] = None
+    ) -> dict | StandardizedUser:
+        api_user: dict = self.client.get_user(user_identifier)
+
+        if standardize:
+            return _standardize_user(api_user)
+        else:
+            return api_user
+
     def get_teams(
         self, standardized_organization: StandardizedOrganization, limit: Optional[int] = None
     ) -> Generator[StandardizedTeam, None, None]:
@@ -267,7 +277,8 @@ class GithubAdapter(GitAdapter):
                     if repo_meta_data.latest_commit_date_on_default_branch <= pull_since:
                         logging_helper.send_to_agent_log_file(
                             f'Not pulling commits for branch {branch_name} because there are no commits newer than the pull_since data. '
-                            f'pull_since: {pull_since}, latest_commit_for_branch: {repo_meta_data.latest_commit_date_on_default_branch}'
+                            f'pull_since: {pull_since}, latest_commit_for_branch: {repo_meta_data.latest_commit_date_on_default_branch}',
+                            level=logging.DEBUG,
                         )
                         continue
 
@@ -504,6 +515,41 @@ class GithubAdapter(GitAdapter):
     def git_provider_pr_endpoint_supports_date_filtering(self):
         return False
 
+    def get_pr(
+        self,
+        login: str,
+        repo_id: str,
+        pr_number: str,
+        standardize: bool = False,
+    ) -> dict | StandardizedPullRequest:
+        """Get a single pull request by its number within a repository.
+
+        Args:
+            login (str): The organization login
+            repo_id (str): The repository NAME, NOT THE ID
+            pr_number (int): The pull request number
+            standardize (bool, optional): Whether to return a standardized object or raw API data. Defaults to False.
+
+        Returns:
+            StandardizedPullRequest | dict: The standardized pull request object or raw API data
+        """
+        api_pr = self.client.get_pr(
+            login=login,
+            repo_name=repo_id,
+            pr_number=pr_number,
+        )
+
+        if standardize:
+            standardized_repo: StandardizedRepository = self.get_repo(
+                login, repo_id, standardize=True  # type: ignore
+            )
+            return _standardize_pr(
+                api_pr,
+                standardized_repo,
+            )
+        else:
+            return api_pr
+
     def get_prs(
         self,
         standardized_repo: StandardizedRepository,
@@ -521,13 +567,15 @@ class GithubAdapter(GitAdapter):
             repo_metadata = self.repo_id_to_repo_meta_data[repo_id]
             if not repo_metadata.latest_pr_updated_at:
                 logging_helper.send_to_agent_log_file(
-                    f'Not querying for PR data, because the repo meta for {standardized_repo.name} indicates that there are no PRs'
+                    f'Not querying for PR data, because the repo meta for {standardized_repo.name} indicates that there are no PRs',
+                    level=logging.DEBUG,
                 )
                 return
             if start_window and repo_metadata.latest_pr_updated_at <= start_window:
                 logging_helper.send_to_agent_log_file(
                     f'Not querying for PR data, because the repo meta for {standardized_repo.name} indicates that the latest PR '
-                    f'was updated on {repo_metadata.latest_pr_updated_at} before our pull from window {start_window}'
+                    f'was updated on {repo_metadata.latest_pr_updated_at} before our pull from window {start_window}',
+                    level=logging.DEBUG,
                 )
                 return
 
@@ -612,6 +660,92 @@ class GithubAdapter(GitAdapter):
         ):
             yield pr_number, _get_standardized_pr_comments(api_comments)
 
+    def get_repo(
+        self, login: str, repo_id: str, standardize: bool = False
+    ) -> StandardizedRepository | dict:
+        """Get a single repository by its ID within an organization.
+
+        Args:
+            login (str): The organization login
+            repo_id (str): The repository NAME, NOT THE ID
+
+        Returns:
+            StandardizedRepository: The standardized repository object
+        """
+        api_repo = self.client.get_repo(
+            login=login,
+            repo_name=repo_id,
+        )
+        standardized_organization = _standardize_organization(
+            api_org=self.client.get_organization_by_login(login)
+        )
+
+        if standardize:
+            return _standardize_repo(
+                api_repo=api_repo,
+                standardized_organization=standardized_organization,
+            )
+        else:
+            return api_repo
+
+    def get_commit(
+        self, login: str, repo_id: str, commit_hash: str, standardize: bool = False
+    ) -> dict | StandardizedCommit:
+        """Get a single commit by it's hash. You must know the organization login and repo ID as well.
+        If you are missing one of these values you could try to get the repo first via get_repo and then
+        reference the values from the StandardizedRepository object
+
+        Args:
+            login (str): The organization login
+            repo_id (str): The repository NAME, NOT THE ID
+            commit_hash (str): The commit hash
+            standardized (bool, optional): Whether to return a standardized object or raw API data. Defaults to False.
+        """
+        standardized_repo: StandardizedRepository = self.get_repo(login, repo_id, standardize=True)  # type: ignore
+
+        api_commit: dict = self.client.get_commit(
+            login=login,
+            repo_name=standardized_repo.name,
+            sha=commit_hash,
+        )
+
+        if standardize:
+            return _standardize_commit(
+                api_commit=api_commit,
+                standardized_repo=standardized_repo,
+                branch_name='',  # branch name is unknown in this context
+            )
+        else:
+            return api_commit
+
+    def get_branch(self, login, repo_id, branch_name, standardize=False):
+        """Get a single branch by its name. You must know the organization login and repo ID as well.
+        If you are missing one of these values you could try to get the repo first via get_repo and then
+        reference the values from the StandardizedRepository object
+
+        Args:
+            login (str): The organization login
+            repo_id (str): The repository NAME, NOT THE ID
+            branch_name (str): The branch name
+            standardized (bool, optional): Whether to return a standardized object or raw API data. Defaults to False.
+        """
+
+        api_branch: dict = self.client.get_branch(
+            login=login,
+            repo_name=repo_id,
+            branch_name=branch_name,
+        )
+
+        if standardize:
+            standardized_repo: StandardizedRepository = self.get_repo(login, repo_id, standardized=True)  # type: ignore
+            return _standardize_branch(
+                api_branch=api_branch,
+                repo_id=standardized_repo.id,
+                is_default_branch=standardized_repo.default_branch_name == branch_name,
+            )
+        else:
+            return api_branch
+
 
 '''
 
@@ -717,7 +851,7 @@ def _standardize_commit(
     api_commit: Dict,
     standardized_repo: StandardizedRepository,
     branch_name: str,
-):
+) -> StandardizedCommit:
     author = _standardize_user(api_commit['author'])
     commit_url = api_commit['url']
     return StandardizedCommit(
@@ -774,7 +908,7 @@ def _get_standardized_reviews(api_reviews: List[Dict]):
 def _standardize_pr(
     api_pr: Dict,
     standardized_repo: StandardizedRepository,
-):
+) -> StandardizedPullRequest:
     base_branch_name = api_pr['baseRefName']
     head_branch_name = api_pr['headRefName']
     standardized_merge_commit = (
