@@ -1,9 +1,9 @@
 import logging
 from datetime import datetime
-from typing import Any, Dict, Generator, List, Optional, Union
+from typing import Any, Dict, Generator, List, Optional, Union, cast
 
 from gitlab.base import RESTObject
-from gitlab.v4.objects import Group, Project, ProjectBranch, ProjectCommit
+from gitlab.v4.objects import Project, ProjectBranch, ProjectCommit
 from gitlab.v4.objects import User as GitlabUser
 
 from jf_ingest import logging_helper
@@ -152,6 +152,16 @@ class GitlabAdapter(GitAdapter):
             url=api_user.web_url,
         )
 
+    @staticmethod
+    def _standardize_user_from_json(api_user: dict) -> StandardizedUser:
+        return StandardizedUser(
+            id=api_user.get('id', ''),
+            name=api_user.get('name', ''),
+            login=api_user.get('username', ''),
+            email=None,
+            url=api_user.get('web_url'),
+        )
+
     @staticmethod
     def _standardize_gitlab_pull_request_gql(
         gitlab_pr: dict,
@@ -413,6 +423,16 @@ class GitlabAdapter(GitAdapter):
 
             yield self._standardize_user(user)
 
+    def get_user(
+        self, user_identifier: str, standardize: bool = False, org_login: Optional[str] = None
+    ) -> dict | StandardizedUser:
+        api_user = self.client.get_user(user_identifier)
+
+        if standardize:
+            return self._standardize_user_from_json(api_user)
+        else:
+            return api_user
+
     def get_teams(
         self, standardized_organization: StandardizedOrganization, limit: Optional[int] = None
     ) -> Generator[StandardizedTeam, None, None]:
@@ -454,6 +474,38 @@ class GitlabAdapter(GitAdapter):
             if limit and idx >= limit:
                 return
 
+    def get_repo(
+        self,
+        login: str,  # Not actually used for Gitlab
+        repo_id: str,
+        standardize: bool = False,
+    ) -> StandardizedRepository | dict:
+        """Get a single repository by its ID
+
+        Args:
+            repo_id (str): The ID of the repository to retrieve
+
+        Returns:
+            StandardizedRepository: A standardized repository object
+        """
+        api_repo = self.client.get_repo(repo_id)
+
+        if standardize:
+            # Need to build a standardized organization object to pass into the standardization function
+            standardized_organization = StandardizedOrganization(
+                id=str(api_repo.namespace['id']),
+                name=api_repo.namespace['name'],
+                login=str(api_repo.namespace['id']),
+                url='',
+            )
+            return self._standardize_gitlab_project(
+                api_repo,
+                standardized_organization,
+            )
+        else:
+
+            return cast(dict, vars(api_repo)['_attrs'])
+
     def get_repos_count(
         self, standardized_organization: StandardizedOrganization, only_private: bool = False
     ) -> int:
@@ -694,6 +746,35 @@ class GitlabAdapter(GitAdapter):
             if limit and j >= limit:
                 break
 
+    def get_pr(
+        self,
+        login: str,
+        repo_id: str,
+        pr_number: str,
+        standardize: bool = False,
+    ) -> dict:
+        """Get a single PR by its ID
+
+        Args:
+            login (str): The login of the organization the repo belongs to
+            repo_id (str): The ID of the repository the PR belongs to
+            pr_id (str): The ID of the PR to retrieve
+
+        Returns:
+            dict: The raw API response for the PR
+        """
+
+        standardized_repo = self.get_repo(login, repo_id, standardize=True)
+        api_pr = self.client.get_pr(standardized_repo.full_path, pr_number)  # type: ignore
+
+        if standardize:
+            return self._standardize_gitlab_pull_request_gql(
+                api_pr,
+                standardized_repo,  # type: ignore[arg-type]
+            )  # type: ignore[return-value]
+
+        return api_pr  # type: ignore
+
     def get_prs(
         self,
         standardized_repo: StandardizedRepository,
@@ -822,3 +903,72 @@ class GitlabAdapter(GitAdapter):
 
             if limit and i >= limit:
                 break
+
+    def get_commit(
+        self, login: str, repo_id: str, commit_hash: str, standardize: bool = False
+    ) -> dict | StandardizedCommit:
+        """Get a single commit by it's hash. You must know the organization login and repo ID as well.
+        If you are missing one of these values you could try to get the repo first via get_repo and then
+        reference the values from the StandardizedRepository object
+
+        Args:
+            login (str): The login of the organization the repo belongs to
+            repo_id (str): The ID of the repository the commit belongs to
+            commit_hash (str): The hash of the commit to retrieve
+            standardized (bool): When True, return a StandardizedCommit object.
+                When False, return the raw API response. Defaults to False.
+        """
+        api_commit = self.client.get_commit(
+            repo_id=repo_id,
+            commit_hash=commit_hash,
+        )
+
+        if standardize:
+            # Need to build a standardized repo object to pass into the standardization function
+            api_repo = self.client.get_repo(repo_id)
+            standardized_organization = StandardizedOrganization(
+                id=login,
+                name='',
+                login=login,
+                url='',
+            )
+            standardized_repo = self._standardize_gitlab_project(
+                api_repo,
+                standardized_organization,
+            )
+            return self._standardize_gitlab_commit(api_commit, standardized_repo, branch_name='')
+        else:
+            return cast(dict, vars(api_commit)['_attrs'])
+
+    def get_branch(
+        self, login, repo_id, branch_name, standardize=False
+    ) -> dict | StandardizedBranch:
+        """Get a single branch by its name"""
+        api_branch = self.client.get_branch(
+            repo_id=repo_id,
+            branch_name=branch_name,
+        )
+
+        if standardize:
+            # Need to build a standardized repo object to pass into the standardization function
+            api_repo = self.client.get_repo(repo_id)
+            standardized_organization = StandardizedOrganization(
+                id=login,
+                name='',
+                login=login,
+                url='',
+            )
+            standardized_repo = self._standardize_gitlab_project(
+                api_repo,
+                standardized_organization,
+            )
+
+            if branch := self._standardize_gitlab_branch(
+                api_branch,
+                standardized_repo,
+            ):
+                return branch
+            else:
+                raise ValueError(f'Branch {branch_name} could not be standardized.')
+        else:
+            return cast(dict, vars(api_branch)['_attrs'])
