@@ -1,14 +1,19 @@
+import logging
 from datetime import datetime
 from typing import Dict, Generator, List, Optional, Tuple
 
 import requests
+from requests.adapters import HTTPAdapter
 
 from jf_ingest import logging_helper
 from jf_ingest.config import AzureDevopsAuthConfig, GitAuthConfig
+from jf_ingest.jf_git.exceptions import GitAuthenticationException
 from jf_ingest.utils import retry_for_status, retry_session
 
 ADO_DEFAULT_API_URL = 'https://dev.azure.com'
 
+logger = logging.getLogger(__name__)
+
 
 class AzureDevopsClient:
     '''
@@ -33,7 +38,7 @@ class AzureDevopsClient:
             raise Exception(
                 f'Provided Auth Config for AzureDevops Clients is {type(git_auth_config)}, but type AzureDevopsAuthConfig is required'
             )
-        self.base_url = git_auth_config.base_url or ADO_DEFAULT_API_URL
+        self.base_url = (git_auth_config.base_url or ADO_DEFAULT_API_URL).strip('/')
         self.token = git_auth_config.token
         self.session = git_auth_config.session or retry_session(**kwargs)
         self.api_version = git_auth_config.api_version
@@ -43,6 +48,11 @@ class AzureDevopsClient:
                 'api-version': self.api_version,
             }
 
+        # Increase our adapter pool size. This will likely only matter for async runs.
+        adapter = HTTPAdapter(pool_connections=30, pool_maxsize=100, pool_block=True)
+        self.session.mount('http://', adapter)
+        self.session.mount('https://', adapter)
+
         # Basic authentication, using no username and the PAT as the password
         self.session.auth = ('', self.token)
         self.session.verify = True
@@ -73,10 +83,19 @@ class AzureDevopsClient:
             continuation_token = result.headers.get('x-ms-continuationtoken')
             return result, continuation_token
         except requests.exceptions.HTTPError as e:
-            if e.response.status_code == 404 and ignore404:
-                logging_helper.send_to_agent_log_file(f'Caught a 404 for {url} - ignoring')
-                return None, None
-            raise
+            if e.response.status_code == 404:
+                if ignore404:
+                    logging_helper.send_to_agent_log_file(f'Caught a 404 for {url} - ignoring')
+                    return None, None
+                else:
+                    raise GitAuthenticationException(
+                        f'404 error returned from ADO API for url {url} with ignore404={ignore404}. '
+                        'This usually indicates a problem with the authentication token, which may '
+                        'have expired or had its permissions revoked.',
+                        original_exception=e,
+                    )
+            else:
+                raise e
 
     @staticmethod
     def format_datetime_to_ado_str(dt: datetime) -> str:
@@ -109,7 +128,19 @@ class AzureDevopsClient:
         )
         result, continuation_token = raw_result_tuple
         if result:
-            return result.json(), continuation_token
+            try:
+                return result.json(), continuation_token
+            except requests.exceptions.JSONDecodeError as e:
+                content_type = (result.headers.get('Content-Type') or '').lower()
+
+                if 'application/json' not in content_type:
+                    raise GitAuthenticationException(
+                        'Non-JSON response from ADO API. This usually indicates a problem with the authentication token,'
+                        f' which may have expired or had its permissions revoked. Returned content type: {content_type}',
+                        original_exception=e,
+                    )
+                else:
+                    raise e
         else:
             return {}, None
 
@@ -259,6 +290,18 @@ class AzureDevopsClient:
         url = f'{self.base_url}/{org_name}/_apis/git/repositories?api-version={self.api_version}'
         return self.get_single_page(url)
 
+    def get_repo(self, org_name: str, repo_id: str) -> Dict:
+        """Get a single repository from the ADO API
+
+        Args:
+            org_name (str): The organization to query for
+            repo_id (str): The repository ID to query for
+        Returns:
+            Dict: A single repo dictionary from ADO
+        """
+        url = f'{self.base_url}/{org_name}/_apis/git/repositories/{repo_id}?api-version={self.api_version}'
+        return self.get_single_object(url)
+
     def get_repos_count(self, org_name: str) -> int:
         """Get the count of repositories from the ADO API
 
@@ -289,6 +332,23 @@ class AzureDevopsClient:
             params={'api-version': f'{self.api_version}-preview.1'},
         )
 
+    def get_graph_user(self, org_name: str, user_identifier: str) -> Dict:
+        """Get a single user from the ADO API
+
+        Args:
+            org_name (str): The org to query for
+            user_identifier (str): The user ID or principal name to query for
+        Returns:
+            Dict: A raw User dictionary object
+        """
+        url = f"{self.base_url}/{org_name}/_apis/graph/users/{user_identifier}"
+        return self.get_single_object(
+            # XXX: This API is not yet GA? Hopefully down the road we'll be able to do away
+            # with the URL and api-version overrides
+            url.replace('dev.azure.com', 'vssps.dev.azure.com'),
+            params={'api-version': f'{self.api_version}-preview.1'},
+        )
+
     def get_teams(self, org_name: str) -> Generator[Dict, None, None]:
         """A Generator for getting team data from the ADO API
 
@@ -371,7 +431,7 @@ class AzureDevopsClient:
 
         return self.get_all_pages_using_skip_and_top(url, params)
 
-    def get_commit(self, org_name: str, project_name: str, repo_id: str, commit_id: str) -> Dict:
+    def get_commit(self, org_name: str, project_name: str, repo_id: str, commit_hash: str) -> Dict:
         """Get a single commit
 
         Args:
@@ -379,14 +439,47 @@ class AzureDevopsClient:
             project_name (str): A Project name. A project name is extracted from the repo URL. See _project_name_from_repo
             repo_id (str): A Repo ID
             branch_name (str): A Branch Name
-            commit_id (str): a commit ID to search for
+            commit_hash (str): a commit ID to search for
 
         Returns:
             Dict: A raw Commit Dictionary
         """
-        url = f"{self.base_url}/{org_name}/{project_name}/_apis/git/repositories/{repo_id}/commits/{commit_id}?api-version={self.api_version}"
+        url = f"{self.base_url}/{org_name}/{project_name}/_apis/git/repositories/{repo_id}/commits/{commit_hash}?api-version={self.api_version}"
         return self.get_single_object(url)
 
+    def get_branch(
+        self,
+        org_name: str,
+        project_name: str,
+        repo_id: str,
+        branch_name: str,
+    ) -> Dict:
+        """Get a single branch
+
+        Args:
+            org_name (str): An org name
+            project_name (str): A Project name. A project name is extracted from the repo URL. See _project_name_from_repo
+            repo_id (str): A repository ID
+            branch_name (str): A branch name
+
+        Returns:
+            Dict: A raw branch in the form of a dictionary
+        """
+        url = f"{self.base_url}/{org_name}/{project_name}/_apis/git/repositories/{repo_id}/refs/heads/{branch_name}?api-version={self.api_version}"
+
+        result = self.get_single_object(url)
+        if result['count'] == 0:
+            raise Exception(f'Branch {branch_name} not found in repo {repo_id}')
+        elif result['count'] > 1:
+            logger.error(
+                f'Multiple branches found for {branch_name} in repo {repo_id}. Full response: {result}'
+            )
+            raise Exception(
+                f'Multiple branches found for {branch_name} in repo {repo_id}. See logs for complete response.'
+            )
+        else:
+            return self.get_single_object(url)['value'][0]  # type: ignore
+
     def get_branches(
         self,
         org_name: str,
@@ -450,6 +543,26 @@ class AzureDevopsClient:
         }
         return self.get_all_pages_using_skip_and_top(url, params, result_container='changes')
 
+    def get_pull_request(
+        self,
+        org_name: str,
+        project_name: str,
+        repo_id: str,
+        pr_id: int,
+    ) -> Dict:
+        """Get a single pull request dictionary, pulled from the ADO API
+
+        Args:
+            org_name (str): An organization name
+            project_name (str): A Project name. A project name is extracted from the repo URL. See _project_name_from_repo
+            repo_id (str): A repository ID
+            pr_id (int): A pull request ID
+        Returns:
+            Dict: A raw pull request dictionary
+        """
+        url = f"{self.base_url}/{org_name}/{project_name}/_apis/git/repositories/{repo_id}/pullrequests/{pr_id}?api-version={self.api_version}"
+        return self.get_single_object(url)
+
     def get_pull_requests(
         self,
         org_name: str,
