@@ -8,6 +8,7 @@ from typing import Any, Dict, Generator, List, Optional, Tuple, Union, cast
 import jwt
 import requests
 from dateutil import parser
+from requests.adapters import HTTPAdapter
 from requests.utils import default_user_agent
 
 from jf_ingest.config import GitAuthConfig, GithubAuthConfig
@@ -79,6 +80,13 @@ class GithubClient:
     """
     GITHUB_GQL_SHORT_REPO_FRAGMENT = "... on Repository {name, id:databaseId, url}"
 
+    GITHUB_GQL_BRANCH_REF_FRAGMENT = f"""
+        ... on Ref {{
+            name
+            target {{sha: oid}}
+        }}
+    """
+
     # The PR query is HUGE, we shouldn't query more than about 25 at a time
     MAX_PAGE_SIZE_FOR_PR_QUERY = 25
 
@@ -132,6 +140,10 @@ class GithubClient:
                 )
 
         self.session = github_auth_config.session or retry_session(**kwargs)
+        # Increase our adapter pool size. This will likely only matter for async runs.
+        adapter = HTTPAdapter(pool_connections=30, pool_maxsize=100, pool_block=True)
+        self.session.mount('http://', adapter)
+        self.session.mount('https://', adapter)
         self.session.verify = True
         self.session.headers.update(
             {
@@ -596,6 +608,14 @@ class GithubClient:
             for user in page['data']['organization']['userQuery']['users']:
                 yield user
 
+    def get_user(self, user_identifier: str) -> dict:
+        url = f'{self.rest_api_url}/users/{user_identifier}'
+        result = retry_for_status(
+            self.session.get, url, statuses_to_retry=GITHUB_STATUSES_TO_RETRY_ON
+        )
+        result.raise_for_status()
+        return result.json()  # type: ignore
+
     def get_team_members(self, login: str, team_slug: str) -> Generator[dict, None, None]:
         query_body = f"""{{
             {GQL_RATELIMIT_INFO_BLOCK}
@@ -704,10 +724,7 @@ class GithubClient:
                         branchQuery: refs(refPrefix:"refs/heads/", first: 100, after: %s) {{
                             {GQL_PAGE_INFO_BLOCK}
                             branches: nodes {{
-                                ... on Ref {{
-                                    name
-                                    target {{sha: oid}}
-                                }}
+                                {self.GITHUB_GQL_BRANCH_REF_FRAGMENT}
                             }}
                         }}
                     }}
@@ -721,6 +738,30 @@ class GithubClient:
             for api_branch in page['data']['organization']['repository']['branchQuery']['branches']:
                 yield api_branch
 
+    def get_branch(self, login: str, repo_name: str, branch_name: str) -> dict:
+        query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
+            organization(login: "{login}") {{
+                repository(name: "{repo_name}") {{
+                    ... on Repository {{
+                        branch: ref(qualifiedName: "{branch_name}") {{
+                            {self.GITHUB_GQL_BRANCH_REF_FRAGMENT}
+                        }}
+                    }}
+                }}
+            }}
+        }}
+        """
+        result = self.get_raw_result(query_body)
+        api_branch: dict = result['data']['organization']['repository']['branch']
+        if not api_branch:
+            raise GitHubGQLNotFoundError(
+                error_response=[
+                    {'message': f'Branch with name {branch_name} not found in repo {repo_name}'}
+                ]
+            )
+        return cast(dict, api_branch)
+
     def get_commits_count(
         self, login: str, repo_name: str, branch_name: str, since: datetime
     ) -> int:
@@ -792,6 +833,72 @@ class GithubClient:
                 api_commit['author'] = self._process_git_actor_gql_object(api_commit['author'])
                 yield api_commit
 
+    def get_repo(self, login: str, repo_name: str) -> dict:
+        query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
+            organization(login: "{login}") {{
+                repo: repository(name: "{repo_name}") {{
+                    ... on Repository {{
+                        id: databaseId
+                        name
+                        fullName: nameWithOwner
+                        url
+                        isFork
+                        defaultBranch: defaultBranchRef {{ name, target {{ sha: oid }} }}
+                    }}
+                }}
+            }}
+        }}
+        """
+        try:
+
+            result = self.get_raw_result(query_body)
+            api_repo: dict = result['data']['organization']['repo']
+        except GitHubGQLNotFoundError:
+            message = f'Repo with id {repo_name} not found'
+            try:
+                int(repo_name)
+                warning_message = (
+                    f'Attempted to get repo by name {repo_name}, but the repo name was detected as being an Integer. '
+                    'Github is unlike the other git providers, where we must uniquely identify the repo by it\'s name and it\'s organization login. '
+                    'If you are trying to get a repo by it\'s ID, please attempt to use its name instead.'
+                )
+                message += f'. {warning_message}'
+                logger.warning(warning_message)
+            except ValueError:
+                pass
+
+            raise GitHubGQLNotFoundError(error_response=[{'message': message}])
+
+        return api_repo
+
+    def get_commit(self, login: str, repo_name: str, sha: str) -> dict:
+        query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
+            organization(login: "{login}") {{
+                repo: repository(name: "{repo_name}") {{
+                    ... on Repository {{
+                        commit: object(oid: "{sha}") {{
+                            ... on Commit {{
+                                {self.GITHUB_GQL_COMMIT_FRAGMENT}
+                            }}
+                        }}
+                    }}
+                }}
+            }}
+        }}
+        """
+        result = self.get_raw_result(query_body)
+        api_commit: dict = result['data']['organization']['repo']['commit']
+        if not api_commit:
+            raise GitHubGQLNotFoundError(
+                error_response=[{'message': f'Commit with sha {sha} not found in repo {repo_name}'}]
+            )
+
+        # Overwrite Author block for backwards compatibility
+        api_commit['author'] = self._process_git_actor_gql_object(api_commit['author'])
+        return cast(dict, api_commit)
+
     #
     # PR Queries are HUGE, so pull out reusable blocks (comments, reviews, commits, etc)
     #
@@ -1034,13 +1141,13 @@ class GithubClient:
                             pr_num_str = err.get('path', [])[-1]
                             pr_num = int(pr_num_str.split('_')[-1])
 
-                            logger.info(f'Removing PR {pr_num} from query')
+                            logger.debug(f'Removing PR {pr_num} from query')
                             current_pr_numbers.remove(pr_num)
                         except Exception as e:
                             logger.warning(f'Error removing PR {err} from query: {e}')
 
                     if not current_pr_numbers:
-                        logger.info('No PRs left to query, stopping')
+                        logger.debug('No PRs left to query, stopping')
                         continue_search = False
                         process_authors = False
                 except Exception as e:
@@ -1144,13 +1251,13 @@ class GithubClient:
                         try:
                             pr_num_str = err.get('path', [])[-1]
                             pr_num = int(pr_num_str.split('_')[-1])
-                            logger.info(f'Removing PR {pr_num} from query')
+                            logger.debug(f'Removing PR {pr_num} from query')
                             current_pr_numbers.remove(pr_num)
                         except Exception as e:
                             logger.warning(f'Error removing PR {err} from query: {e}')
 
                     if not current_pr_numbers:
-                        logger.info('No PRs left to query, stopping')
+                        logger.debug('No PRs left to query, stopping')
                         continue_search = False
                         process_batch = False
                 except Exception as e:
@@ -1196,17 +1303,40 @@ class GithubClient:
 
         return pr_review_authors
 
-    def _need_to_fetch_with_deeper_reviews_query(self, api_pr: dict) -> bool:
-        if _gql_query_has_next_page(api_pr['reviewsQuery']):
-            logger.info(f'Need to query more reviews for PR {api_pr["number"]}')
+    def _need_to_fetch_with_deeper_reviews_query(
+        self,
+        api_pr: dict,
+        pr_number: int,
+        repo_name: str,
+    ) -> bool:
+        """Inspects the api_pr dictionary object and determines if we are potentially
+        missing data by seeing if the subqueries (reviews, comments, reactions) have
+        more pages to fetch. The PR Number and Repo Name arguments are used only for
+        logging.
+
+        Args:
+            api_pr (dict): The PR dictionary object returned from the GQL query
+            pr_number (int): The number of the PR being inspected (only for logging purposes)
+            repo_name (str): The name of the repo where the PR is from (only for logging purposes)
+
+        Returns:
+            bool: returns True if we need to make additional review objects
+        """
+        number_and_repo_str = f'PR Number: {pr_number}, Repo: {repo_name}'
+        if not (reviews_query := api_pr.get('reviewsQuery', {})):
+            logger.warning(f'No reviewsQuery found in api_pr for {number_and_repo_str}')
+            return False
+
+        if _gql_query_has_next_page(reviews_query):
+            logger.debug(f'Need to query more reviews for {number_and_repo_str}')
             return True
-        for review in api_pr['reviewsQuery']['reviews']:
-            if _gql_query_has_next_page(review['commentsQuery']):
-                logger.info(f'Need to query more comments for PR {api_pr["number"]}')
+        for review in reviews_query.get('reviews', []):
+            if _gql_query_has_next_page(review.get('commentsQuery', {})):
+                logger.debug(f'Need to query more comments for {number_and_repo_str}')
                 return True
-            for comment in review['commentsQuery']['comments']:
+            for comment in review.get('commentsQuery', {}).get('comments', []):
                 if _gql_query_has_next_page(comment.get('reactionsQuery', {})):
-                    logger.info(f'Need to query more reactions for PR {api_pr["number"]}')
+                    logger.debug(f'Need to query more reactions for {number_and_repo_str}')
                     return True
         return False
 
@@ -1225,22 +1355,13 @@ class GithubClient:
             else list(api_pr['commentsQuery']['comments'])
         )
 
-    def _get_pr_query(
+    def _get_pr_gql_block(
         self,
-        page_size: int,
-        login: str,
-        repo_name: str,
-        include_force_push_commits: bool,
-        pull_files_for_pr: bool,
-        include_top_level_comments: bool,
+        pull_files_for_pr: bool = False,
+        include_top_level_comments: bool = False,
+        include_force_push_commits: bool = False,
     ) -> str:
-        return f"""{{
-            {GQL_RATELIMIT_INFO_BLOCK}
-            organization(login: "{login}") {{
-                repo: repository(name: "{repo_name}") {{
-                    prQuery: pullRequests(first: {page_size}, orderBy: {{direction: DESC, field: UPDATED_AT}}, after: %s) {{
-                        {GQL_PAGE_INFO_BLOCK}
-                        prs: nodes {{
+        return f"""
                             ... on PullRequest {{
                                 id: number
                                 number
@@ -1275,6 +1396,50 @@ class GithubClient:
                                 {self._get_pr_files_query_block(page_size=100, enable_paging=False) if pull_files_for_pr else ''}
                                 {self._get_labels_query_block(page_size=50, enable_paging=False)}
                             }}
+        """
+
+    def get_pr(self, login: str, repo_name: str, pr_number: str) -> dict:
+        query_body = f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
+            organization(login: "{login}") {{
+                repo: repository(name: "{repo_name}") {{
+                    pr: pullRequest(number: {pr_number}) {{
+                        {self._get_pr_gql_block()}
+                    }}
+                }}
+            }}
+        }}
+        """
+        result = self.get_raw_result(query_body=query_body)
+        api_pr: dict = result['data']['organization']['repo']['pr']
+        if not api_pr:
+            raise GitHubGQLNotFoundError(
+                error_response=[
+                    {'message': f'PR with number {pr_number} not found in repo {repo_name}'}
+                ]
+            )
+
+        return self._process_pr_from_gql(
+            api_pr=api_pr, login=login, repo_name=repo_name, repository_label_node_ids_to_id={}
+        )
+
+    def _get_pr_query(
+        self,
+        page_size: int,
+        login: str,
+        repo_name: str,
+        include_force_push_commits: bool,
+        pull_files_for_pr: bool,
+        include_top_level_comments: bool,
+    ) -> str:
+        return f"""{{
+            {GQL_RATELIMIT_INFO_BLOCK}
+            organization(login: "{login}") {{
+                repo: repository(name: "{repo_name}") {{
+                    prQuery: pullRequests(first: {page_size}, orderBy: {{direction: DESC, field: UPDATED_AT}}, after: %s) {{
+                        {GQL_PAGE_INFO_BLOCK}
+                        prs: nodes {{
+                            {self._get_pr_gql_block(include_force_push_commits=include_force_push_commits, pull_files_for_pr=pull_files_for_pr, include_top_level_comments=include_top_level_comments)}
                         }}
                     }}
                 }}
@@ -1282,6 +1447,108 @@ class GithubClient:
         }}
         """
 
+    def _process_pr_from_gql(
+        self,
+        api_pr: dict,
+        login: str,
+        repo_name: str,
+        repository_label_node_ids_to_id: dict,
+        include_force_push_commits: bool = False,
+        include_top_level_comments: bool = False,
+        pull_files_for_pr: bool = False,
+        hash_files_for_prs: bool = False,
+    ) -> dict:
+        # Process and add related PR data (comments, reviews, commits)
+        # This may require additional API calls
+        pr_number = api_pr['number']
+
+        # Load reviews first because we use them in both reviews and comments
+        api_pr['reviews'] = (
+            [r for r in self.get_pr_reviews(login, repo_name, pr_number=pr_number)]
+            if self._need_to_fetch_with_deeper_reviews_query(
+                api_pr, pr_number=pr_number, repo_name=repo_name
+            )
+            else api_pr['reviewsQuery']['reviews']
+        )
+
+        # NOTE: COMMENTS ARE WEIRD! They exist in there own API endpoint (these
+        # are typically top level comments in a PR, considered an IssueComment)
+        # but there are also comments associated with each review (typically only one)
+        # The baseline for what we care about is the Review Level comment, pulled from
+        # the reviews endpoint. Grabbing Top Level Comments is an optional feature flag
+        # Grab the comments pulled from reviews. We ALWAYS want these!
+        comments = []
+        comments.extend(self.get_pr_comments_from_reviews(api_pr['reviews']))
+
+        if include_top_level_comments:
+            # If we need to page for top level comments, do so
+            comments.extend(self.get_top_level_comments(api_pr, login, repo_name, pr_number))
+
+        api_pr['comments'] = comments
+
+        api_pr['commits'] = (
+            [
+                commit
+                for commit in self.get_pr_commits(
+                    login,
+                    repo_name,
+                    pr_number=pr_number,
+                    include_force_push_commits=include_force_push_commits,
+                )
+            ]
+            if api_pr['commitsQuery']['pageInfo']['hasNextPage']
+            else [
+                commit['commit'] if 'commit' in commit else commit['beforeCommit']
+                for commit in api_pr['commitsQuery']['commits']
+                if 'commit' in commit or 'beforeCommit' in commit
+            ]
+        )
+
+        # Do some extra processing on commits to clean up their weird author block
+        for commit in api_pr['commits']:
+            commit['author'] = self._process_git_actor_gql_object(commit['author'])
+
+        if api_pr['mergeCommit'] and api_pr['mergeCommit']['author']:
+            api_pr['mergeCommit']['author'] = self._process_git_actor_gql_object(
+                api_pr['mergeCommit']['author']
+            )
+
+        labels = (
+            [label for label in self.get_pr_labels(login, repo_name, pr_number=pr_number)]
+            if api_pr['labelsQuery']['pageInfo']['hasNextPage']
+            else [label for label in api_pr['labelsQuery']['labels']]
+        )
+        api_pr['labels'] = []
+        # Only add to labels if we have the proper label ID
+        for label in labels:
+            if repository_label_node_ids_to_id.get(label['node_id']):
+                label['id'] = repository_label_node_ids_to_id.get(label['node_id'])
+                api_pr['labels'].append(label)
+
+        # NOTE: Processing files requires quite a bit of in place transformation
+        if pull_files_for_pr:
+            files = api_pr['filesQuery']['files']
+            api_pr['files'] = {}
+            # If there are more files to fetch, fetch them
+            if api_pr['filesQuery']['pageInfo']['hasNextPage']:
+                files = self.get_pr_files(login, repo_name, pr_number=pr_number)
+
+            for file_dict in files:
+                # File path is the dictionary key, and should not be included in the dictionary body
+                file_path = (
+                    hash_filename(file_dict.pop('path'))
+                    if hash_files_for_prs
+                    else file_dict.pop('path')
+                )
+                # The legacy REST API includes a 'changed' field, which is the total of additions and deletions.
+                # To match legacy logic, perform that operations here
+                file_dict['changes'] = file_dict['additions'] + file_dict['deletions']
+                # The legacy REST API has the status enums all lowercase
+                file_dict['status'] = file_dict['status'].lower()
+                api_pr['files'][file_path] = file_dict
+
+        return api_pr
+
     # PR query is HUGE, see above GITHUB_GQL_PR_* blocks for reused code
     # page_size is optimally variable. Most repos only have a 0 to a few PRs day to day,
     # so sometimes the optimal page_size is 0. Generally, we should never go over 25
@@ -1313,97 +1580,17 @@ class GithubClient:
             cursor=transformed_start_cursor,
         ):
             for api_pr in page['data']['organization']['repo']['prQuery']['prs']:
-                # Process and add related PR data (comments, reviews, commits)
-                # This may require additional API calls
-                pr_number = api_pr['number']
-
-                # Load reviews first because we use them in both reviews and comments
-                api_pr['reviews'] = (
-                    [r for r in self.get_pr_reviews(login, repo_name, pr_number=pr_number)]
-                    if self._need_to_fetch_with_deeper_reviews_query(api_pr)
-                    else api_pr['reviewsQuery']['reviews']
+                yield self._process_pr_from_gql(
+                    api_pr=api_pr,
+                    login=login,
+                    repo_name=repo_name,
+                    repository_label_node_ids_to_id=repository_label_node_ids_to_id,
+                    include_force_push_commits=include_force_push_commits,
+                    include_top_level_comments=include_top_level_comments,
+                    pull_files_for_pr=pull_files_for_pr,
+                    hash_files_for_prs=hash_files_for_prs,
                 )
 
-                # NOTE: COMMENTS ARE WEIRD! They exist in there own API endpoint (these
-                # are typically top level comments in a PR, considered an IssueComment)
-                # but there are also comments associated with each review (typically only one)
-                # The baseline for what we care about is the Review Level comment, pulled from
-                # the reviews endpoint. Grabbing Top Level Comments is an optional feature flag
-                # Grab the comments pulled from reviews. We ALWAYS want these!
-                comments = []
-                comments.extend(self.get_pr_comments_from_reviews(api_pr['reviews']))
-
-                if include_top_level_comments:
-                    # If we need to page for top level comments, do so
-                    comments.extend(
-                        self.get_top_level_comments(api_pr, login, repo_name, pr_number)
-                    )
-
-                api_pr['comments'] = comments
-
-                api_pr['commits'] = (
-                    [
-                        commit
-                        for commit in self.get_pr_commits(
-                            login,
-                            repo_name,
-                            pr_number=pr_number,
-                            include_force_push_commits=include_force_push_commits,
-                        )
-                    ]
-                    if api_pr['commitsQuery']['pageInfo']['hasNextPage']
-                    else [
-                        commit['commit'] if 'commit' in commit else commit['beforeCommit']
-                        for commit in api_pr['commitsQuery']['commits']
-                        if 'commit' in commit or 'beforeCommit' in commit
-                    ]
-                )
-
-                # Do some extra processing on commits to clean up their weird author block
-                for commit in api_pr['commits']:
-                    commit['author'] = self._process_git_actor_gql_object(commit['author'])
-
-                if api_pr['mergeCommit'] and api_pr['mergeCommit']['author']:
-                    api_pr['mergeCommit']['author'] = self._process_git_actor_gql_object(
-                        api_pr['mergeCommit']['author']
-                    )
-
-                labels = (
-                    [label for label in self.get_pr_labels(login, repo_name, pr_number=pr_number)]
-                    if api_pr['labelsQuery']['pageInfo']['hasNextPage']
-                    else [label for label in api_pr['labelsQuery']['labels']]
-                )
-                api_pr['labels'] = []
-                # Only add to labels if we have the proper label ID
-                for label in labels:
-                    if repository_label_node_ids_to_id.get(label['node_id']):
-                        label['id'] = repository_label_node_ids_to_id.get(label['node_id'])
-                        api_pr['labels'].append(label)
-
-                # NOTE: Processing files requires quite a bit of in place transformation
-                if pull_files_for_pr:
-                    files = api_pr['filesQuery']['files']
-                    api_pr['files'] = {}
-                    # If there are more files to fetch, fetch them
-                    if api_pr['filesQuery']['pageInfo']['hasNextPage']:
-                        files = self.get_pr_files(login, repo_name, pr_number=pr_number)
-
-                    for file_dict in files:
-                        # File path is the dictionary key, and should not be included in the dictionary body
-                        file_path = (
-                            hash_filename(file_dict.pop('path'))
-                            if hash_files_for_prs
-                            else file_dict.pop('path')
-                        )
-                        # The legacy REST API includes a 'changed' field, which is the total of additions and deletions.
-                        # To match legacy logic, perform that operations here
-                        file_dict['changes'] = file_dict['additions'] + file_dict['deletions']
-                        # The legacy REST API has the status enums all lowercase
-                        file_dict['status'] = file_dict['status'].lower()
-                        api_pr['files'][file_path] = file_dict
-
-                yield api_pr
-
     def get_pr_top_level_comments(
         self, login: str, repo_name: str, pr_number: int
     ) -> Generator[dict, None, None]:
@@ -1689,7 +1876,22 @@ class GithubClient:
                 }}
             }}
             """
-            result = self.get_raw_result(query_body=query_body)
+            try:
+                result = self.get_raw_result(query_body=query_body)
+            except GitHubGQLNotFoundError as e:
+                error_message = str(e)
+                # HACK: If the repo is specifically not findable, skip the entire batch
+                if 'Could not resolve to a Repository with the name' in error_message:
+                    logger.warning(
+                        f'Error message appears to be related to the repository not existing. Skipping batch of PRs'
+                    )
+                    return
+                continue
+            except Exception as e:
+                logger.warning(
+                    f'Error fetching PR comments and reactions for repo {repo_name}: {e}'
+                )
+                continue
             for pr_number in pr_number_batch:
                 api_pr = result['data']['organization']['repo'].get(f'pr_{pr_number}')
                 if not api_pr:
@@ -1698,7 +1900,9 @@ class GithubClient:
 
                 reviews = (
                     [r for r in self.get_pr_reviews(login, repo_name, pr_number=pr_number)]
-                    if self._need_to_fetch_with_deeper_reviews_query(api_pr)
+                    if self._need_to_fetch_with_deeper_reviews_query(
+                        api_pr, pr_number=pr_number, repo_name=repo_name
+                    )
                     else api_pr['reviewsQuery']['reviews']
                 )
 
