@@ -12,6 +12,7 @@ from gitlab.base import RESTObject
 from gitlab.exceptions import GitlabError
 from gitlab.v4.objects import Group, Project, ProjectBranch, ProjectCommit
 from gitlab.v4.objects import User as GitlabUser
+from requests.adapters import HTTPAdapter
 
 from jf_ingest import logging_helper
 from jf_ingest.config import GitAuthConfig, GitLabAuthConfig
@@ -23,7 +24,6 @@ from jf_ingest.jf_git.exceptions import (
 )
 from jf_ingest.jf_git.standardized_models import (
     StandardizedOrganization,
-    StandardizedPullRequest,
     StandardizedRepository,
 )
 from jf_ingest.utils import (
@@ -75,7 +75,17 @@ class GitlabClient:
                     'User-Agent': f'{Constants.JELLYFISH_USER_AGENT} ({requests.utils.default_user_agent()})',
                 }
             )
+        # Increase our adapter pool size. This will likely only matter for async runs.
+        adapter = HTTPAdapter(pool_connections=30, pool_maxsize=100, pool_block=True)
+        self.session.mount('http://', adapter)
+        self.session.mount('https://', adapter)
+
         self.session.verify = auth_config.verify
+
+        # Disable warnings if we aren't verifying SSL. This is very noisy otherwise.
+        if not self.session.verify:
+            requests.packages.urllib3.disable_warnings()
+
         self.client: Gitlab = Gitlab(
             url=self.rest_api_url,
             session=self.session,
@@ -412,9 +422,13 @@ class GitlabClient:
         branch_name: Optional[str] = None,
     ) -> Generator[ProjectCommit | RESTObject, None, None]:
         project_id = jf_repo.id
-        project = retry_for_status(
-            self.client.projects.get, project_id, statuses_to_retry=GITLAB_STATUSES_TO_RETRY
-        )
+        try:
+            project = retry_for_status(
+                self.client.projects.get, project_id, statuses_to_retry=GITLAB_STATUSES_TO_RETRY
+            )
+        except (requests.exceptions.ConnectionError, GitlabError, RetryLimitExceeded) as e:
+            logger.warning(f"Failed to pull project {jf_repo.name}. Skipping. Exception: {e}")
+            return
 
         kwargs_dict: dict[str, Any] = {}
         if branch_name:
@@ -433,6 +447,38 @@ class GitlabClient:
                 f"Failed to pull commits for repo {jf_repo.name}. Skipping. Exception: {e}"
             )
 
+    def get_commit(
+        self,
+        repo_id: str,
+        commit_hash: str,
+    ):
+        project = retry_for_status(
+            self.client.projects.get, repo_id, statuses_to_retry=GITLAB_STATUSES_TO_RETRY
+        )
+        return retry_for_status(
+            project.commits.get, commit_hash, statuses_to_retry=GITLAB_STATUSES_TO_RETRY
+        )
+
+    def get_branch(
+        self,
+        repo_id: str,
+        branch_name: str,
+    ):
+        project = retry_for_status(
+            self.client.projects.get, repo_id, statuses_to_retry=GITLAB_STATUSES_TO_RETRY
+        )
+        return retry_for_status(
+            project.branches.get, branch_name, statuses_to_retry=GITLAB_STATUSES_TO_RETRY
+        )
+
+    def get_repo(
+        self,
+        repo_id: str,
+    ):
+        return retry_for_status(
+            self.client.projects.get, repo_id, statuses_to_retry=GITLAB_STATUSES_TO_RETRY
+        )
+
     def get_branches_for_repo(
         self, jf_repo: StandardizedRepository, branch_name: Optional[str] = None
     ) -> Generator[ProjectBranch | RESTObject, None, None]:
@@ -448,9 +494,14 @@ class GitlabClient:
             ProjectBranch | RESTObject: Objects generated by the GitLab client.
         """
         project_id = jf_repo.id
-        project = retry_for_status(
-            self.client.projects.get, project_id, statuses_to_retry=GITLAB_STATUSES_TO_RETRY
-        )
+        try:
+            project = retry_for_status(
+                self.client.projects.get, project_id, statuses_to_retry=GITLAB_STATUSES_TO_RETRY
+            )
+        except (requests.exceptions.ConnectionError, GitlabError, RetryLimitExceeded) as e:
+            logger.warning(f"Failed to pull project {jf_repo.name}. Skipping. Exception: {e}")
+            return
+
         branches_list: List[ProjectBranch | RESTObject] = []
 
         kwargs_dict: dict[str, Any] = {}
@@ -584,6 +635,110 @@ class GitlabClient:
                 level=logging.WARNING,
             )
 
+    def get_user(self, user_identifer: str) -> dict:
+        url = f'{self.rest_api_url}/api/v4/users/{user_identifer}'
+        result = retry_for_status(self.session.get, url, statuses_to_retry=GITLAB_STATUSES_TO_RETRY)
+        result.raise_for_status()
+        return result.json()  # type: ignore
+
+    def _pr_gql_block(self):
+        return f"""
+    
+                    id
+                    iid
+                    {'closedAt' if not self.compatibility_config['gql_skip_pr_closed_at'] else ''}
+                    updatedAt
+                    mergedAt
+                    createdAt
+                    title
+                    description
+                    webUrl
+                    sourceBranch
+                    targetBranch
+                    mergeCommitSha
+                    diffStatsSummary {{ additions, deletions, fileCount }}
+                    sourceProject {{ {self.GITLAB_GQL_SHORT_REPO_NODES} }}
+                    targetProject {{ {self.GITLAB_GQL_SHORT_REPO_NODES} }}
+                    author {{ {self.GITLAB_GQL_USER_NODES} }}
+                    mergeUser {{ {self.GITLAB_GQL_USER_NODES} }}
+                    {self._get_pr_commits_query_block(enable_paging=False)}
+                    {self._get_pr_comments_query_block(enable_paging=False)}
+                    {self._get_pr_approvals_query_block(enable_paging=False)}
+        """
+
+    def _process_pr_from_gql(
+        self,
+        pr_node: dict,
+        project_full_path: str,
+        pull_files_for_pr: Optional[bool] = True,
+        hash_files_for_pr: Optional[bool] = False,
+    ) -> dict:
+        pr_iid = pr_node['iid']
+
+        # Fetch any commits not fetched on initial page
+        if self.get_path_value('commits.pageInfo.hasNextPage', pr_node):
+            commits = [
+                commit
+                for commit in self.get_pr_commits(
+                    project_full_path=project_full_path,
+                    pr_iid=pr_iid,
+                )
+            ]
+            pr_node['commits']['nodes'] = commits
+
+        # Fetch any comments not fetched on initial page
+        if self.get_path_value('notes.pageInfo.hasNextPage', pr_node):
+            comments = [
+                comment
+                for comment in self.get_pr_comments(
+                    project_full_path=project_full_path,
+                    pr_iid=pr_iid,
+                )
+            ]
+            pr_node['notes']['nodes'] = comments
+
+        # Fetch any approvals not fetched on initial page
+        if self.get_path_value('approvedBy.pageInfo.hasNextPage', pr_node):
+            approvals = [
+                approval
+                for approval in self.get_pr_approvals(
+                    project_full_path=project_full_path,
+                    pr_iid=pr_iid,
+                )
+            ]
+            pr_node['approvedBy']['nodes'] = approvals
+
+        # Hash filenames if enabled and we have file paths to hash
+        if pull_files_for_pr and hash_files_for_pr:
+            hashed_files = []
+            for unhashed_file in pr_node.get('diffStats', []):
+                unhashed_file['path'] = hash_filename(unhashed_file['path'])
+                hashed_files.append(unhashed_file)
+            pr_node['diffStats'] = hashed_files
+
+        return pr_node
+
+    def get_pr(
+        self,
+        project_full_path: str,
+        pr_number: str,
+    ):
+        query_body = f"""   
+        {{         
+            project(fullPath: "{project_full_path}") {{
+                mergeRequest(iid: "{pr_number}") {{
+                    {self._pr_gql_block()}
+                }}
+            }}
+        }}
+        """
+        result = self.get_raw_result_gql(query_body=query_body)
+        pr_node = self.get_path_value('data.project.mergeRequest', result)
+        return self._process_pr_from_gql(
+            pr_node,  # type: ignore
+            project_full_path=project_full_path,
+        )
+
     def get_prs(
         self,
         project_full_path: str,
@@ -616,27 +771,7 @@ class GitlabClient:
                 mergeRequests(first: %s, after: %s, sort: UPDATED_DESC{f', updatedAfter: "{start_window.isoformat()}"' if start_window else ''}{f', updatedBefore: "{end_window.isoformat()}"' if end_window else ''}) {{
                     {GQL_PAGE_INFO_BLOCK}
                     nodes {{
-                        id
-                        iid
-                        {'closedAt' if not self.compatibility_config['gql_skip_pr_closed_at'] else ''}
-                        updatedAt
-                        mergedAt
-                        createdAt
-                        title
-                        description
-                        webUrl
-                        sourceBranch
-                        targetBranch
-                        mergeCommitSha
-                        diffStatsSummary {{ additions, deletions, fileCount }}
-                        {'diffStats { additions, deletions, path }' if pull_files_for_pr else ''}
-                        sourceProject {{ {self.GITLAB_GQL_SHORT_REPO_NODES} }}
-                        targetProject {{ {self.GITLAB_GQL_SHORT_REPO_NODES} }}
-                        author {{ {self.GITLAB_GQL_USER_NODES} }}
-                        mergeUser {{ {self.GITLAB_GQL_USER_NODES} }}
-                        {self._get_pr_commits_query_block(enable_paging=False)}
-                        {self._get_pr_comments_query_block(enable_paging=False)}
-                        {self._get_pr_approvals_query_block(enable_paging=False)}
+                        {self._pr_gql_block()}
                     }}
                 }}
             }}
@@ -651,50 +786,12 @@ class GitlabClient:
         ):
             pr_nodes = self.get_path_value('data.project.mergeRequests.nodes', page) or []
             for pr_node in pr_nodes:
-                pr_iid = pr_node['iid']
-
-                # Fetch any commits not fetched on initial page
-                if self.get_path_value('commits.pageInfo.hasNextPage', pr_node):
-                    commits = [
-                        commit
-                        for commit in self.get_pr_commits(
-                            project_full_path=project_full_path,
-                            pr_iid=pr_iid,
-                        )
-                    ]
-                    pr_node['commits']['nodes'] = commits
-
-                # Fetch any comments not fetched on initial page
-                if self.get_path_value('notes.pageInfo.hasNextPage', pr_node):
-                    comments = [
-                        comment
-                        for comment in self.get_pr_comments(
-                            project_full_path=project_full_path,
-                            pr_iid=pr_iid,
-                        )
-                    ]
-                    pr_node['notes']['nodes'] = comments
-
-                # Fetch any approvals not fetched on initial page
-                if self.get_path_value('approvedBy.pageInfo.hasNextPage', pr_node):
-                    approvals = [
-                        approval
-                        for approval in self.get_pr_approvals(
-                            project_full_path=project_full_path,
-                            pr_iid=pr_iid,
-                        )
-                    ]
-                    pr_node['approvedBy']['nodes'] = approvals
-
-                # Hash filenames if enabled and we have file paths to hash
-                if pull_files_for_pr and hash_files_for_pr:
-                    hashed_files = []
-                    for unhashed_file in pr_node.get('diffStats', []):
-                        unhashed_file['path'] = hash_filename(unhashed_file['path'])
-                        hashed_files.append(unhashed_file)
-                    pr_node['diffStats'] = hashed_files
-
-                yield pr_node
+                yield self._process_pr_from_gql(
+                    pr_node,
+                    project_full_path=project_full_path,
+                    pull_files_for_pr=pull_files_for_pr,
+                    hash_files_for_pr=hash_files_for_pr,
+                )
 
     def get_pr_commits(
         self,
