@@ -33,6 +33,7 @@ from jf_ingest.jf_jira.downloaders import (
     download_users,
     download_users_by_urls,
     download_worklogs,
+    fetch_id_to_key_for_all_existing,
     get_issue_ids_for_rekeys_and_deletes,
     get_jira_search_batch_size,
     is_jql_enhanced_search_available,
@@ -245,15 +246,11 @@ def pull_issues_wrapper(
     ###########################################################################
     #
     # STEP TWO [Optional]: Pull issues by their updated date
+    # NOTE: Here we still only pull Issue IDs so that we can
+    # bulk pull all issues by IDs in step 3
     #
     ###########################################################################
-    issues_by_updated_date_results: IssueDownloadingResult = IssueDownloadingResult(
-        downloaded_ids=set(),
-        issue_ids_too_large_to_upload=set(),
-        discovered_parent_ids=set(),
-        total_batches=0,
-        users_found=set(),
-    )
+    issue_ids_updated_by_date: set[str] = set()
     if jira_config.pull_issues_by_date:
         project_id_to_pull_from: dict[str, datetime] = {
             str(proj["id"]): (
@@ -265,45 +262,35 @@ def pull_issues_wrapper(
             if proj["key"] not in jira_config.exclude_projects
         }
         logger.info(
-            f'Attempting to pull all the full Jira Issues that have been updated since our last Jira Download across {len(project_id_to_pull_from)} projects.'
-        )
-        issues_by_updated_date_generator = pull_all_jira_issues_by_date(
-            jira_connection=jira_connect_or_fallback_connection,
-            project_key_to_pull_from=project_id_to_pull_from,
-            num_parallel_threads=jira_config.issue_download_concurrent_threads,
-            batch_size=jira_issues_batch_size,
-            jql_project_batch_size=50,
-            jql_filter=jira_config.issue_jql_filter,
-            expand_fields=["changelog"],
-            include_fields=include_fields,
-            exclude_fields=exclude_fields,
-            use_jql_enhanced_search=use_jql_enhanced_search,
+            f'Attempting to pull all the Jira Issue IDs that have been updated since our last Jira Download across {len(project_id_to_pull_from)} projects.'
         )
+
         with record_span('pull_jira_issues_by_date'):
-            issues_by_updated_date_results = load_issues_in_batches(
-                issues_by_updated_date_generator,
-                ingest_io_helper,
-                ingest_config,
-                ingest_config.jira_config,
-            )
-            issue_ids_not_uploaded.update(
-                issues_by_updated_date_results.issue_ids_too_large_to_upload
+            issue_id_and_key_by_updated = fetch_id_to_key_for_all_existing(
+                jira_connection=jira_connect_or_fallback_connection,
+                project_ids=list(project_id_to_pull_from.keys()),
+                pull_from=jira_config.pull_from,
+                jql_filter=jira_config.issue_jql_filter,
+                use_jql_enhanced_search=use_jql_enhanced_search,
+                project_id_to_pull_from=project_id_to_pull_from,
             )
+            issue_ids_updated_by_date = set(issue_id_and_key_by_updated.keys())
+
             logger.info(
-                f'We have downloaded {len(issues_by_updated_date_results.downloaded_ids)} issues that have been updated or created since our last pull across {len(project_id_to_pull_from)} projects.'
+                f'We have found {len(issue_ids_updated_by_date)} issue IDs that have been updated or created since our last pull across {len(project_id_to_pull_from)} projects.'
             )
 
-    file_batch_cursor = issues_by_updated_date_results.total_batches
     ###########################################################################
     #
     # Step 3: Download the full Jira Issues for potentially rekeys and/or missing issues.
     # Also redownload issues that have been marked for redownload in our system
     #
     ###########################################################################
+    # Extend the list of IDs to download with any issues that have been updated by date
+    ids_to_download.update(issue_ids_updated_by_date)
     # Extend this list with issues that need to be redownloaded for Jellyfish reasons
     ids_to_download.update(jira_config.jellyfish_issue_ids_for_redownload)
-    # Next, exclude any IDs that we have just done a download for, via the updated date
-    ids_to_download = ids_to_download.difference(issues_by_updated_date_results.downloaded_ids)
+
     logging_helper.send_to_agent_log_file(
         f'We need to download an additional {len(ids_to_download)} issues because they have been rekeyed or marked for redownload by Jellyfish'
     )
@@ -325,18 +312,14 @@ def pull_issues_wrapper(
             ingest_io_helper,
             ingest_config,
             ingest_config.jira_config,
-            batch_number_start=file_batch_cursor,
         )
-        file_batch_cursor += issues_by_ids_results.total_batches
         issue_ids_not_uploaded.update(issues_by_ids_results.issue_ids_too_large_to_upload)
     # Now, combine all issue IDs that we've downloaded
     all_downloaded_ids: set[str] = set()
-    all_downloaded_ids.update(issues_by_updated_date_results.downloaded_ids)
     all_downloaded_ids.update(issues_by_ids_results.downloaded_ids)
 
     # Then, combine the parents of the issues we downloaded by update date, and issues we downloaded by ID
     all_discovered_parent_ids: set[str] = set()
-    all_discovered_parent_ids.update(issues_by_updated_date_results.discovered_parent_ids)
     all_discovered_parent_ids.update(issues_by_ids_results.discovered_parent_ids)
 
     # Now, find unique parents that we have NOT downloaded and that we need to
@@ -346,7 +329,6 @@ def pull_issues_wrapper(
 
     # Track all unique users across all issues we're downloading
     users_found_across_all_issues: set[UserMetadata] = set()
-    users_found_across_all_issues.update(issues_by_updated_date_results.users_found)
     users_found_across_all_issues.update(issues_by_ids_results.users_found)
 
     ###########################################################################
@@ -361,6 +343,7 @@ def pull_issues_wrapper(
     ###########################################################################
     with record_span('pull_jira_issues_parents'):
         depth_level = 1
+        file_batch_cursor = issues_by_ids_results.total_batches
         while to_be_downloaded_parent_ids:
             logging_helper.send_to_agent_log_file(
                 f'Attempting to pull more parents ({len(to_be_downloaded_parent_ids)} detected parents to pull). Some of them are {sorted(list(to_be_downloaded_parent_ids))[:30]} Depth level: {depth_level}. Any parents before {jira_config.pull_from} will not be downloaded'
