@@ -24,6 +24,7 @@ from jira.exceptions import JIRAError
 from requests.adapters import HTTPAdapter
 from requests.cookies import RequestsCookieJar, cookiejar_from_dict
 from tqdm import tqdm
+from tqdm.asyncio import tqdm as atqdm
 from urllib3 import Retry
 
 from jf_ingest import logging_helper
@@ -609,7 +610,7 @@ class TqdmToLogger(io.StringIO):
         self.logger.log(self.level, self.buf)
 
 
-def tqdm_to_logger(*args, **kwargs) -> tqdm:
+def tqdm_to_logger(*args, use_async=False, **kwargs) -> tqdm:
     """HACK helper function to wrap up the TQDM progress bar to extend it's functionality.
     If you use this helper function in place of the normal tqdm() call it will behave differently
     in Agent and in Direct Connect. In Agent, TQDM will act as if you called it directly.
@@ -621,13 +622,14 @@ def tqdm_to_logger(*args, **kwargs) -> tqdm:
     """
     from jf_ingest.config import IngestionType
 
+    tqdm_func = tqdm if not use_async else atqdm
     ingestion_type = get_ingestion_type()
     if ingestion_type == IngestionType.DIRECT_CONNECT:
         tqdm_out = TqdmToLogger(logger, level=logging.INFO)
         # Log progress bar every 10 minutes (minimum) in prefect
         mininterval = kwargs.get('mininterval', 60 * 10)
-        return tqdm(file=tqdm_out, mininterval=mininterval, ascii=False, *args, **kwargs)
-    return tqdm(*args, **kwargs)
+        return tqdm_func(file=tqdm_out, mininterval=mininterval, ascii=False, *args, **kwargs)
+    return tqdm_func(*args, **kwargs)
 
 
 class ThreadPoolExecutorWithLogging(ThreadPoolExecutor):
@@ -971,6 +973,16 @@ def get_id_from_gid(gitlab_gid: str, object_name: str) -> str:
     return gitlab_gid.split(f'gid://gitlab/{object_name}/')[1]
 
 
+def generate_gid_from_id(object_id: str, object_name: str) -> str:
+    """Helper function.
+    The Gitlab GQL requires many IDs with this weird GID format.
+    All we care about is the number trailing at the end.
+    Objects are capitalized (Group, Project, MergeRequest, User, etc...)
+    Gitlab Format: gid://gitlab/{object_name}/{ID_NUMBER}
+    """
+    return f'gid://gitlab/{object_name}/{object_id}'
+
+
 def normalize_datetime(dt: Any, tz: timezone) -> Any:
     if type(dt) is datetime:
         if dt.year == 1:
