@@ -5,14 +5,30 @@ from typing import Generator
 import pytest
 import requests
 import requests_mock
+import time
 
 from jf_ingest.config import AzureDevopsAuthConfig
-from jf_ingest.jf_git.clients.azure_devops import AzureDevopsClient
+from jf_ingest.jf_git.clients.azure_devops import ADO_DEFAULT_API_URL, AzureDevopsClient
+from jf_ingest.jf_git.exceptions import GitAuthenticationException
+
+import jf_ingest.jf_git.clients.azure_devops as ado_client_mod
+import jf_ingest.utils as utils_mod
 
 TEST_BASE_URL = 'https://ado.com'
 CONTINUATION_TOKEN_HEADER_SLUG = 'x-ms-continuationtoken'
 
 
+@pytest.fixture(autouse=True)
+def disable_retries(monkeypatch):
+    monkeypatch.setattr(time, "sleep", lambda *_: None)
+    monkeypatch.setattr(utils_mod, "get_wait_time", lambda e, r: 0)
+
+    def passthrough(f, *args, **kwargs):
+        return f(*args, **kwargs)
+
+    monkeypatch.setattr(ado_client_mod, "retry_for_status", passthrough)
+
+
 def _get_client(api_version: str = None):
     session = requests.Session()
     auth_config = AzureDevopsAuthConfig(
@@ -25,6 +41,70 @@ def _get_client(api_version: str = None):
         auth_config.api_version = api_version
     return AzureDevopsClient(auth_config)
 
+def test_url_formatting():
+    session = requests.Session()
+    auth_config = AzureDevopsAuthConfig(
+        company_slug='company_slug',
+        token='let-me-in',
+        session=session,
+    )
+    client = AzureDevopsClient(auth_config)
+    assert client.base_url == ADO_DEFAULT_API_URL, "No URL Provided yields default"
+    
+    auth_config = AzureDevopsAuthConfig(
+        company_slug='company_slug',
+        token='let-me-in',
+        base_url=TEST_BASE_URL,
+        session=session,
+    )
+    client = AzureDevopsClient(auth_config)
+    assert client.base_url == TEST_BASE_URL, "Base URL matches base url"
+    
+    auth_config = AzureDevopsAuthConfig(
+        company_slug='company_slug',
+        token='let-me-in',
+        base_url=f'{ADO_DEFAULT_API_URL}/',
+        session=session,
+    )
+    client = AzureDevopsClient(auth_config)
+    assert client.base_url == ADO_DEFAULT_API_URL, "Base URL matches base url (test two)"
+    
+    auth_config = AzureDevopsAuthConfig(
+        company_slug='company_slug',
+        token='let-me-in',
+        base_url=f'{TEST_BASE_URL}/',
+        session=session,
+    )
+    client = AzureDevopsClient(auth_config)
+    assert client.base_url == TEST_BASE_URL, "Base URL matches base url (slash got stripped)"
+    
+    auth_config = AzureDevopsAuthConfig(
+        company_slug='company_slug',
+        token='let-me-in',
+        base_url=f'{TEST_BASE_URL}///',
+        session=session,
+    )
+    client = AzureDevopsClient(auth_config)
+    assert client.base_url == TEST_BASE_URL, "Base URL matches base url (multiple slashes got stripped)"
+    
+    auth_config = AzureDevopsAuthConfig(
+        company_slug='company_slug',
+        token='let-me-in',
+        base_url=f'https://gavin.com/////',
+        session=session,
+    )
+    client = AzureDevopsClient(auth_config)
+    assert client.base_url == 'https://gavin.com', "Base URL matches base url (multiple slashes got stripped)"
+    
+    auth_config = AzureDevopsAuthConfig(
+        company_slug='company_slug',
+        token='let-me-in',
+        base_url=f'https://gav.in/phitt/',
+        session=session,
+    )
+    client = AzureDevopsClient(auth_config)
+    assert client.base_url == 'https://gav.in/phitt', "Base URL matches base url (multiple slashes got stripped)"
+
 def test_format_datetime_to_ado_str():
     dt = datetime(2024, 11, 12, 4, 30, 0)
     assert AzureDevopsClient.format_datetime_to_ado_str(dt) == '2024-11-12'
@@ -64,8 +144,10 @@ def test_get_json(requests_mock: requests_mock.Mocker):
     # Test that API version works with non default value
     _test_get_json(404, ignore_404=True)
 
-    with pytest.raises(requests.exceptions.HTTPError):
+    with pytest.raises(GitAuthenticationException):
         _test_get_json(404, ignore_404=False)
+
+    with pytest.raises(requests.exceptions.HTTPError):
         _test_get_json(500)
 
 
@@ -101,8 +183,10 @@ def test_get_single_object(requests_mock: requests_mock.Mocker):
     # Test that API version works with non default value
     _test_get_json(404, ignore_404=True)
 
-    with pytest.raises(requests.exceptions.HTTPError):
+    with pytest.raises(GitAuthenticationException):
         _test_get_json(404, ignore_404=False)
+
+    with pytest.raises(requests.exceptions.HTTPError):
         _test_get_json(500)
 
 
