@@ -287,3 +287,180 @@ def test_page_results_gql(mocker, requests_mock: requests_mock.Mocker):
 
     _test_page_results_gql_intter(items_per_page=1, number_of_pages=10)
     _test_page_results_gql_intter(items_per_page=10, number_of_pages=100)
+
+
+def test_need_to_fetch_more_reviews(mocker, requests_mock: requests_mock.Mocker, client: GithubClient):
+    need_to_fetch_deeper = client._need_to_fetch_with_deeper_reviews_query(
+        api_pr={},
+        pr_number=1,
+        repo_name='test/repo',
+    )
+    assert need_to_fetch_deeper is False, "Expected False when no reviewQuery present"
+    
+    need_to_fetch_deeper = client._need_to_fetch_with_deeper_reviews_query(
+        api_pr={
+            'reviewsQuery': {
+                'pageInfo': {
+                    'hasNextPage': True,
+                    'endCursor': 'cursor_1',
+                },
+                'reviews': [],
+            }
+        },
+        pr_number=1,
+        repo_name='test/repo',
+    )
+    assert need_to_fetch_deeper is True, "Expected True when reviewsQuery has next page"
+    
+    need_to_fetch_deeper = client._need_to_fetch_with_deeper_reviews_query(
+        api_pr={
+            'reviewsQuery': {
+                'pageInfo': {
+                    'hasNextPage': False,
+                    'endCursor': 'cursor_1',
+                },
+                'reviews': [
+                    {
+                        'commentsQuery': {
+                            'pageInfo': {
+                                'hasNextPage': True,
+                                'endCursor': 'cursor_2',
+                            },
+                            'comments': [],
+                        }
+                    }
+                ],
+            }
+        },
+        pr_number=1,
+        repo_name='test/repo',
+    )
+    assert need_to_fetch_deeper is True, "Expected True when a review's commentsQuery has next page"
+    
+    need_to_fetch_deeper = client._need_to_fetch_with_deeper_reviews_query(
+        api_pr={
+            'reviewsQuery': {
+                'pageInfo': {
+                    'hasNextPage': False,
+                    'endCursor': 'cursor_1',
+                },
+                'reviews': [
+                    {
+                        'filler': 'data',
+                    }
+                ],
+            }
+        },
+        pr_number=1,
+        repo_name='test/repo',
+    )
+    assert need_to_fetch_deeper is False, "Expected False when a reviewsQuery has no next page and there are no comments"
+    
+    need_to_fetch_deeper = client._need_to_fetch_with_deeper_reviews_query(
+        api_pr={
+            'reviewsQuery': {
+                'pageInfo': {
+                    'hasNextPage': False,
+                    'endCursor': 'cursor_1',
+                },
+                'reviews': [
+                    {
+                        'commentsQuery': {
+                            'pageInfo': {
+                                'hasNextPage': False,
+                                'endCursor': 'cursor_2',
+                            },
+                            'comments': [
+                                {
+                                    'filler': 'data',
+                                }
+                            ],
+                        }
+                    }
+                ],
+            }
+        },
+        pr_number=1,
+        repo_name='test/repo',
+    )
+    assert need_to_fetch_deeper is False, "Expected False when a comment has no reactionsQuery"
+    
+    need_to_fetch_deeper = client._need_to_fetch_with_deeper_reviews_query(
+        api_pr={
+            'reviewsQuery': {
+                'pageInfo': {
+                    'hasNextPage': False,
+                    'endCursor': 'cursor_1',
+                },
+                'reviews': [
+                    {
+                        'commentsQuery': {
+                            'pageInfo': {
+                                'hasNextPage': False,
+                                'endCursor': 'cursor_2',
+                            },
+                            'comments': [
+                                {
+                                    'reactionsQuery': {
+                                        'pageInfo': {
+                                            'hasNextPage': True,
+                                            'endCursor': 'cursor_3',
+                                        },
+                                        'reactions': [],
+                                    }
+                                }
+                            ],
+                        }
+                    }
+                ],
+            }
+        },
+        pr_number=1,
+        repo_name='test/repo',
+    )
+    assert need_to_fetch_deeper is True, "Expected True when a comment's reactionsQuery has next page"
+    
+    need_to_fetch_deeper = client._need_to_fetch_with_deeper_reviews_query(
+        api_pr={
+            'reviewsQuery': {
+                'pageInfo': {
+                    'hasNextPage': False,
+                    'endCursor': 'cursor_1',
+                },
+                'reviews': [
+                    {
+                        'commentsQuery': {
+                            'pageInfo': {
+                                'hasNextPage': False,
+                                'endCursor': 'cursor_2',
+                            },
+                            'comments': [
+                                {
+                                    'reactionsQuery': {
+                                        'pageInfo': {
+                                            'hasNextPage': False,
+                                            'endCursor': 'cursor_3',
+                                        },
+                                        'reactions': [],
+                                    }
+                                }
+                            ],
+                        }
+                    }
+                ],
+            }
+        },
+        pr_number=1,
+        repo_name='test/repo',
+    )
+    assert need_to_fetch_deeper is False, "Expected False when there are no extra pages to fetch"
+    
+
+def test_need_to_fetch_more_reviews_with_no_pr_number_or_repo_name(mocker, requests_mock: requests_mock.Mocker, client: GithubClient):
+    need_to_fetch_deeper = client._need_to_fetch_with_deeper_reviews_query(
+        api_pr={},
+        pr_number=None,
+        repo_name=None,
+    )
+    assert need_to_fetch_deeper is False, "Expected False when no reviewQuery present, and no exceptions when PR number and Repo Name are None"
+    
\ No newline at end of file
