@@ -41,11 +41,24 @@ from tests.jf_jira.utils import (
     _register_jira_uri,
     _register_jira_uri_with_file,
     get_jira_mock_connection,
+    get_jira_mock_download_config,
 )
 
 logger = logging.getLogger(__name__)
 
 
+def _mock_server_info_endpoints(mocker):
+    """Helper function to mock both v2 and v3 serverInfo endpoints"""
+    server_info_response = '{"baseUrl":"https://test-co.atlassian.net","version":"1001.0.0-SNAPSHOT","versionNumbers":[1001,0,0],"deploymentType":"Cloud"}'
+    _register_jira_uri(mocker, "serverInfo", server_info_response)
+    # Mock v3 serverInfo endpoint
+    mocker.register_uri(
+        'GET',
+        'https://test-co.atlassian.net/rest/api/3/serverInfo',
+        text=server_info_response
+    )
+
+
 def _generate_mock_address_for_issue_jql(
     m: requests_mock.Mocker,
     jql_query: str,
@@ -224,12 +237,12 @@ def _mock_jira_issue_by_ids(
             issues=_generate_issues(id_batch),
             max_results=batch_size,
         )
-        
+
 def test_generate_jql_for_batch_ids():
     ids = ["1", "2", "3", "4", "5"]
     jql_query = generate_jql_for_batch_of_ids(ids)
     assert jql_query == 'id in (1,2,3,4,5) order by id asc'
-        
+
 def test_generate_jql_for_batch_ids_with_pull_from():
     ids = ["1", "2", "3", "4", "5"]
     pull_from = datetime(2024, 12, 29, 17, 20, 1)
@@ -495,7 +508,7 @@ def test_filter_changelogs_inclusion_filtering_by_id():
     for history in filtered_issue["changelog"]["histories"]:
         assert len(history["items"]) == 1
         assert history['items'][0]['fieldId'] == field_id_1
-        
+
 def test_filter_changelogs_inclusion_filtering_by_name():
     field_id_1 = 'FIELD_ID_1'
     field_name_1 = 'FIELD NAME 1'
@@ -526,7 +539,7 @@ def test_filter_changelogs_inclusion_filtering_by_name():
     for history in filtered_issue["changelog"]["histories"]:
         assert len(history["items"]) == 1
         assert history['items'][0]['field'] == field_name_1
-        
+
 def test_filter_changelogs_exclusion_filtering_by_id():
     field_id_1 = 'FIELD_ID_1'
     field_name_1 = 'FIELD NAME 1'
@@ -558,7 +571,7 @@ def test_filter_changelogs_exclusion_filtering_by_id():
     for history in filtered_issue["changelog"]["histories"]:
         assert len(history["items"]) == 1
         assert history['items'][0]['fieldId'] == field_id_1
-        
+
 def test_filter_changelogs_exclusion_filtering_by_name():
     field_id_1 = 'FIELD_ID_1'
     field_name_1 = 'FIELD NAME 1'
@@ -707,7 +720,7 @@ def test_download_issue_page_ensure_error_never_raised():
         side_effect=JIRAError(status_code=500),
     ):
         issues = _download_issue_page(
-            jira_connection=get_jira_mock_connection(), jql_query='', start_at=0, batch_size=100
+            jira_connection=get_jira_mock_connection(), jql_query='', batch_size=100, start_at=0
         )
         assert len(issues) == 0
 
@@ -716,7 +729,7 @@ def test_download_issue_page_ensure_error_never_raised():
         side_effect=Exception('random exception'),
     ):
         issues = _download_issue_page(
-            jira_connection=get_jira_mock_connection(), jql_query='', start_at=0, batch_size=100
+            jira_connection=get_jira_mock_connection(), jql_query='', batch_size=100, start_at=0
         )
         assert len(issues) == 0
 
@@ -736,10 +749,10 @@ def test_download_issue_page_ensure_fields_not_ignored():
                 HTTP_ACTION='POST',
             )
             yield jira_mock_connection, mocker
-    
+
     with _mocked_jira_return(requested_batch_size=0, returned_batch_size=0) as (jira_conn, mocker):
         _download_issue_page(
-            jira_connection=jira_conn, jql_query='', start_at=0, batch_size=0, expand_fields=[], include_fields=[
+            jira_connection=jira_conn, jql_query='', batch_size=0, start_at=0, expand_fields=[], include_fields=[
                 JiraFieldIdentifier(jira_field_id='id', jira_field_name='ID')
             ]
         )
@@ -752,7 +765,7 @@ def test_download_issue_page_ensure_fields_not_ignored():
                 request_body = json.loads(request.body)
                 assert request_body['fields'] == ['id']
         assert post_found
-        
+
 def test_download_issue_page_ensure_fields_not_ignored_more():
     """
     The _download_issue_page should NEVER raise an error.
@@ -769,10 +782,10 @@ def test_download_issue_page_ensure_fields_not_ignored_more():
                 HTTP_ACTION='POST',
             )
             yield jira_mock_connection, mocker
-    
+
     with _mocked_jira_return(requested_batch_size=0, returned_batch_size=0) as (jira_conn, mocker):
         _download_issue_page(
-            jira_connection=jira_conn, jql_query='', start_at=0, batch_size=0, expand_fields=[], include_fields=[
+            jira_connection=jira_conn, jql_query='', batch_size=0, start_at=0, expand_fields=[], include_fields=[
                 JiraFieldIdentifier(jira_field_id='id', jira_field_name='ID'),
                 JiraFieldIdentifier(jira_field_id='customfield_101234', jira_field_name='Sprint'),
                 JiraFieldIdentifier(jira_field_id='customfield_201244', jira_field_name='Thing'),
@@ -826,90 +839,150 @@ def test_generate_project_pull_from_bulk_jql_with_issue_filter():
 def test_is_jql_enhanced_search_available_success():
     """Test that is_jql_enhanced_search_available returns True when /search/jql endpoint returns 200"""
     with requests_mock.Mocker() as mocker:
-        # Mock successful response from /search/jql endpoint
+        # Mock successful response from /search/jql endpoint (v3 API)
         jira_return_val = '{"maxResults": 1, "nextPageToken": "eyJzdGFydEF0IjoxfQ==", "issues": []}'
-        _register_jira_uri(
-            mocker,
-            endpoint="search/jql",
-            return_value=jira_return_val,
-            HTTP_ACTION='POST',
+        mocker.register_uri(
+            'POST',
+            'https://test-co.atlassian.net/rest/api/3/search/jql',
+            text=jira_return_val,
             status_code=200,
         )
         
-        jira_connection = get_jira_mock_connection(mocker)
-        result = is_jql_enhanced_search_available(jira_connection)
+        # Mock serverInfo endpoints for connection creation
+        _mock_server_info_endpoints(mocker)
+        
+        jira_config = get_jira_mock_download_config()
+        result = is_jql_enhanced_search_available(
+            jira_config=jira_config,
+            jql_enhanced_search_enabled=True,
+            force_legacy_api=False
+        )
         assert result is True
 
 
 def test_is_jql_enhanced_search_available_not_found():
     """Test that is_jql_enhanced_search_available returns False when /search/jql endpoint returns 404"""
     with requests_mock.Mocker() as mocker:
-        # Mock 404 response from /search/jql endpoint
-        _register_jira_uri(
-            mocker,
-            endpoint="search/jql",
-            return_value='{"errorMessages":["Not Found"]}',
-            HTTP_ACTION='POST',
+        # Mock 404 response from /search/jql endpoint (v3 API)
+        mocker.register_uri(
+            'POST',
+            'https://test-co.atlassian.net/rest/api/3/search/jql',
+            text='{"errorMessages":["Not Found"]}',
             status_code=404,
         )
         
-        jira_connection = get_jira_mock_connection(mocker)
-        result = is_jql_enhanced_search_available(jira_connection)
+        # Mock serverInfo endpoints for connection creation
+        _mock_server_info_endpoints(mocker)
+        
+        jira_config = get_jira_mock_download_config()
+        result = is_jql_enhanced_search_available(
+            jira_config=jira_config,
+            jql_enhanced_search_enabled=True,
+            force_legacy_api=False
+        )
         assert result is False
 
 
 def test_is_jql_enhanced_search_available_server_error():
     """Test that is_jql_enhanced_search_available returns False when /search/jql endpoint returns 500 (conservative fallback)"""
     with requests_mock.Mocker() as mocker:
-        # Mock 500 response from /search/jql endpoint
-        _register_jira_uri(
-            mocker,
-            endpoint="search/jql",
-            return_value='{"errorMessages":["Internal Server Error"]}',
-            HTTP_ACTION='POST',
+        # Mock 500 response from /search/jql endpoint (v3 API)
+        mocker.register_uri(
+            'POST',
+            'https://test-co.atlassian.net/rest/api/3/search/jql',
+            text='{"errorMessages":["Internal Server Error"]}',
             status_code=500,
         )
         
-        jira_connection = get_jira_mock_connection(mocker)
-        result = is_jql_enhanced_search_available(jira_connection)
+        # Mock serverInfo endpoints for connection creation
+        _mock_server_info_endpoints(mocker)
+        
+        jira_config = get_jira_mock_download_config()
+        result = is_jql_enhanced_search_available(
+            jira_config=jira_config,
+            jql_enhanced_search_enabled=True,
+            force_legacy_api=False
+        )
         assert result is False
 
 
 def test_is_jql_enhanced_search_available_auth_error():
     """Test that is_jql_enhanced_search_available returns False when /search/jql endpoint returns 401 (conservative fallback)"""
     with requests_mock.Mocker() as mocker:
-        # Mock 401 response from /search/jql endpoint
-        _register_jira_uri(
-            mocker,
-            endpoint="search/jql",
-            return_value='{"errorMessages":["Unauthorized"]}',
-            HTTP_ACTION='POST',
+        # Mock 401 response from /search/jql endpoint (v3 API)
+        mocker.register_uri(
+            'POST',
+            'https://test-co.atlassian.net/rest/api/3/search/jql',
+            text='{"errorMessages":["Unauthorized"]}',
             status_code=401,
         )
         
-        jira_connection = get_jira_mock_connection(mocker)
-        result = is_jql_enhanced_search_available(jira_connection)
+        # Mock serverInfo endpoints for connection creation
+        _mock_server_info_endpoints(mocker)
+        
+        jira_config = get_jira_mock_download_config()
+        result = is_jql_enhanced_search_available(
+            jira_config=jira_config,
+            jql_enhanced_search_enabled=True,
+            force_legacy_api=False
+        )
         assert result is False
 
 
 def test_is_jql_enhanced_search_available_network_exception():
     """Test that is_jql_enhanced_search_available returns False when network exception occurs"""
     import requests
-    with requests_mock.Mocker() as mocker:
-        # Mock network exception
-        mocker.register_uri(
-            'POST',
-            'https://test-co.atlassian.net/rest/api/2/search/jql',
-            exc=requests.exceptions.ConnectTimeout
-        )
+    from unittest.mock import MagicMock
+    
+    # Mock get_jira_connection to avoid connection creation delays
+    with patch('jf_ingest.jf_jira.downloaders.get_jira_connection') as mock_get_connection:
+        # Create a mock connection that raises an exception when _session.post is called
+        mock_connection = MagicMock()
+        mock_connection._session.post.side_effect = requests.exceptions.ConnectionError("Connection failed")
+        mock_connection._get_url.return_value = "https://test-co.atlassian.net/rest/api/3/search/jql"
+        mock_get_connection.return_value = mock_connection
         
-        jira_connection = get_jira_mock_connection(mocker)
-        result = is_jql_enhanced_search_available(jira_connection)
+        jira_config = get_jira_mock_download_config()
+        result = is_jql_enhanced_search_available(
+            jira_config=jira_config,
+            jql_enhanced_search_enabled=True,
+            force_legacy_api=False
+        )
         assert result is False
 
 
+def test_is_jql_enhanced_search_available_feature_flag_disabled():
+    """Test that is_jql_enhanced_search_available returns False when jql_enhanced_search_enabled is False"""
+    jira_config = get_jira_mock_download_config()
+    result = is_jql_enhanced_search_available(
+        jira_config=jira_config,
+        jql_enhanced_search_enabled=False,
+        force_legacy_api=False
+    )
+    assert result is False
+
+
+def test_is_jql_enhanced_search_available_force_legacy_api():
+    """Test that is_jql_enhanced_search_available returns False when force_legacy_api is True"""
+    jira_config = get_jira_mock_download_config()
+    result = is_jql_enhanced_search_available(
+        jira_config=jira_config,
+        jql_enhanced_search_enabled=True,
+        force_legacy_api=True
+    )
+    assert result is False
+
+
+def test_is_jql_enhanced_search_available_both_flags_false():
+    """Test that is_jql_enhanced_search_available returns False when both flags are False"""
+    jira_config = get_jira_mock_download_config()
+    result = is_jql_enhanced_search_available(
+        jira_config=jira_config,
+        jql_enhanced_search_enabled=False,
+        force_legacy_api=False
+    )
+    assert result is False
 
-    
 
 def test_post_raw_result_jql_enhanced_success():
     """Test that _post_raw_result_jql_enhanced successfully calls the /search/jql endpoint"""
@@ -1546,17 +1619,17 @@ def test_download_issue_page_with_legacy_api():
             return_value=json.dumps(mock_response),
             HTTP_ACTION='POST',
         )
-        
+
         # Test with use_jql_enhanced_search=False (default)
         result = _download_issue_page(
             jira_connection=jira_mock_connection,
             jql_query="project = TEST",
-            start_at=0,
             batch_size=100,
+            start_at=0,
             return_total=True,
             use_jql_enhanced_search=False
         )
-        
+
         # Should return 2 values for legacy API
         assert isinstance(result, tuple)
         assert len(result) == 2
@@ -1581,18 +1654,18 @@ def test_download_issue_page_with_jql_enhanced_search():
             return_value=json.dumps(mock_response),
             HTTP_ACTION='POST',
         )
-        
+
         # Test with use_jql_enhanced_search=True
         result = _download_issue_page(
             jira_connection=jira_mock_connection,
             jql_query="project = TEST",
-            start_at=0,
             batch_size=100,
+            start_at=0,
             return_total=True,
             use_jql_enhanced_search=True,
             next_page_token=None
         )
-        
+
         # Should return (issues, total) tuple when return_total=True
         assert isinstance(result, tuple)
         assert len(result) == 2
@@ -1617,18 +1690,18 @@ def test_download_issue_page_with_jql_enhanced_search_no_return_total():
             return_value=json.dumps(mock_response),
             HTTP_ACTION='POST',
         )
-        
+
         # Test with use_jql_enhanced_search=True and return_total=False
         result = _download_issue_page(
             jira_connection=jira_mock_connection,
             jql_query="project = TEST",
-            start_at=0,
             batch_size=100,
+            start_at=0,
             return_total=False,
             use_jql_enhanced_search=True,
-            next_page_token=None
+            next_page_token=None,
         )
-        
+
         # Should return just issues list when return_total=False
         assert isinstance(result, list)
         assert len(result) == 1
@@ -1653,15 +1726,15 @@ def test_download_issue_page_backward_compatibility():
             return_value=json.dumps(mock_response),
             HTTP_ACTION='POST',
         )
-        
+
         # Test calling without new parameters (should use legacy API)
         result = _download_issue_page(
             jira_connection=jira_mock_connection,
             jql_query="project = TEST",
+            batch_size=100,
             start_at=0,
-            batch_size=100
         )
-        
+
         # Should return just issues list (no total) for backward compatibility
         assert isinstance(result, list)
         assert len(result) == 1
@@ -1675,12 +1748,12 @@ def test_download_issue_page_error_handling_with_jql_enhanced_search():
         result = _download_issue_page(
             jira_connection=get_jira_mock_connection(),
             jql_query="project = TEST",
-            start_at=1000,  # Set start_at > end_at to trigger error return
             batch_size=100,
+            start_at=1000,
             return_total=True,
-            use_jql_enhanced_search=True
+            use_jql_enhanced_search=True,
         )
-        
+
         # Should return empty results with correct signature (issues, total) when return_total=True
         assert isinstance(result, tuple)
         assert len(result) == 2
@@ -1740,8 +1813,8 @@ def test_get_jira_results_looped_with_jql_enhanced_search():
         jql_query = "project = TEST"
         
         # For this test, let's use a simple single-page scenario to avoid the complexity
-        # Mock single call that returns all results with no next_page_token
-        mock_download.return_value = ([{"id": "1", "key": "TEST-1"}, {"id": "2", "key": "TEST-2"}, {"id": "3", "key": "TEST-3"}], 3, None)
+        # Mock single call that returns all results with next_page_token=None (no more pages)
+        mock_download.return_value = ([{"id": "1", "key": "TEST-1"}, {"id": "2", "key": "TEST-2"}, {"id": "3", "key": "TEST-3"}], None)
         
         results = get_jira_results_looped(
             jira_connection=jira_connection,
@@ -1982,7 +2055,6 @@ def test_get_all_project_issue_counts_backward_compatibility():
         assert "use_jql_enhanced_search" not in call_args[1] or call_args[1]["use_jql_enhanced_search"] is False
 
 
-
 def test_fetch_id_to_key_for_all_existing_parameter_passing():
     """Test that fetch_id_to_key_for_all_existing passes use_jql_enhanced_search parameter through to called functions."""
     from unittest.mock import patch, MagicMock
@@ -2097,4 +2169,4 @@ def test_fetch_id_to_key_for_all_existing_api_selection_integration():
         assert results_call_args[1]['use_jql_enhanced_search'] is True
         
         # Verify return value
-        assert result == {'1': 'PROJ-1'}
\ No newline at end of file
+        assert result == {'1': 'PROJ-1'}
